package com.indicee.api.connector;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.net.URLEncoder;
import java.security.NoSuchAlgorithmException;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.net.ssl.SSLContext;

import oauth.signpost.commonshttp.CommonsHttpOAuthConsumer;

import org.apache.commons.io.IOUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.http.Header;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.MultipartEntity;
import org.apache.http.entity.mime.content.ContentBody;
import org.apache.http.entity.mime.content.InputStreamBody;
import org.apache.http.entity.mime.content.StringBody;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.SingleClientConnManager;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.connector.exception.IndiceeAPIStatusException;
import com.indicee.api.connector.util.InputStreamWrapper;
import com.indicee.api.services.impl.IndiceeAPIService;
import com.indicee.api.services.impl.ReportServiceV3Impl;
import com.indicee.api.services.interfaces.ContributionService;
import com.indicee.api.services.interfaces.ReportServiceV3;
import com.indicee.api.util.XmlHelper;

/**
 * The IndiceeAPIConnector provides methods to make API calls to Indicee. There
 * are methods to make both GET and POST calls. The Connector can either be used
 * directly or used with the {@link IndiceeAPIService}s. For example:
 * <p>
 * 
 * <pre>
 * IndiceeAPIConnector connector = new IndiceeAPIConnector(host, consumerKey, consumerSecret, tokenKey, tokenSecret);
 * {@link ReportServiceV3} reportService = new {@link ReportServiceV3Impl}(connector);
 * </pre>
 * 
 * @author victork
 * 
 */
public class IndiceeAPIConnector implements Serializable {

	private static final long serialVersionUID = 1L;

	private static final Log logger = LogFactory.getLog(IndiceeAPIConnector.class);
	private static final String REMAINING_QUOTA_HEADER = "indicee_api_remaining_quota";
	
	private int remainingQuota = -1; //-1 means unknown
	
	private interface StreamProcessor<R> {
		R call(InputStream is);
	}

	private static final class OutputStreamProcessor implements StreamProcessor<Void> {

		private final OutputStream os;
		
		public OutputStreamProcessor(OutputStream os) {
			super();
			this.os = os;
		}

		/**
		 * @see com.indicee.api.connector.IndiceeAPIConnector.StreamProcessor#call(java.io.InputStream)
		 */
		@Override
		public Void call(InputStream is) {

			try {
				IOUtils.copy(is, os);
			} catch (IOException e) {
				logger.error("Error copying input to output stream", e);
				throw new RuntimeException(e);
			}
			return null;
		}
	}
	
    private static final StreamProcessor<byte[]> byteArrayStreamProcessor = new StreamProcessor<byte[]>() {
    	@Override
		public byte[] call(InputStream is) {
			try {
				return IOUtils.toByteArray(is);
			} catch (IOException e) {
				logger.error(e);
				throw new RuntimeException("Unexpected Exception reading data returned by server.", e);
			}
		}
    };
    
    private static final StreamProcessor<String> stringStreamProcessor = new StreamProcessor<String>() {
		@Override
		public String call(InputStream is) {
			try {
				return IOUtils.toString(is, "UTF-8");
			} catch (IOException e) {
				logger.error(e);
				throw new RuntimeException("Unexpected Exception parsing string returned by server.", e);
			}
		}
    };
    
    private static final StreamProcessor<Document> xmlStreamProcessor = new StreamProcessor<Document>() {
		@Override
		public Document call(InputStream is) {
			try {
				return XmlHelper.readXmlDocumentFromStream(is);
	        } catch (Exception e) {
	        	logger.error(e);
	            throw new RuntimeException("Unexpected Exception parsing xml returned by server.", e);
	        }
		}
	};
    
	//Warning this is not thread safe.
    private final CommonsHttpOAuthConsumer commonsHttpConsumer;
    private final String host;

    /**
     * Creates a new Indicee connector. You can use this class to directly make
     * calls to the Indicee API.
     * <p>
     * The IndiceeAPIConnector is also used when constructing
     * {@link IndiceeAPIService}, such as the {@link ReportServiceV3}, or the
     * {@link ContributionService}.
     * 
     * 
     * @param host
     *            The Indicee server to connect to. For example
     *            "https://secure.indicee.com"
     * @param consumerKey
     *            The consumer key
     * @param consumerSecret
     *            The consumer secret
     * @param tokenKey
     *            The toekn key
     * @param tokenSecret
     *            The token secret
     * 
     * @see IndiceeAPIService
     * @throws IllegalArgumentException
     *             If any of the parameters are null or empty. Also if the host
     *             is not a valid url.
     */
    public IndiceeAPIConnector(String host, String consumerKey, String consumerSecret, String tokenKey,
            String tokenSecret) {
        //validate the arguements
        {
            if (host == null || host.isEmpty()) {
                throw new IllegalArgumentException("The host cannot be null or empty.");
            } else if (consumerKey == null || consumerKey.isEmpty()) {
                throw new IllegalArgumentException("The consumerKey cannot be null or empty.");
            } else if (consumerSecret == null || consumerSecret.isEmpty()) {
                throw new IllegalArgumentException("The consumerSecret cannot be null or empty.");
            } else if (tokenKey == null || tokenKey.isEmpty()) {
                throw new IllegalArgumentException("The tokenKey cannot be null or empty.");
            } else if (tokenSecret == null || tokenSecret.isEmpty()) {
                throw new IllegalArgumentException("The tokenSecret cannot be null or empty.");
            }
            //Test to see if the host url is a valid url.
            try {
                new URL(host);
            } catch (MalformedURLException e) {
            	logger.error(e);
                throw new IllegalArgumentException("The host url is invalid: " + e.getMessage());
            }
        }
        this.host = host;

        // oauth initialization
        commonsHttpConsumer = new CommonsHttpOAuthConsumer(consumerKey, consumerSecret);
        commonsHttpConsumer.setTokenWithSecret(tokenKey, tokenSecret);
    }

    public String getConsumerKey() {
        return commonsHttpConsumer.getConsumerKey();
    }

    public String getConsumerSecret() {
        return commonsHttpConsumer.getConsumerSecret();
    }

    public String getHost() {
        return host;
    }

    public String getTokenKey() {
        return commonsHttpConsumer.getToken();
    }

    public String getTokenSecret() {
        return commonsHttpConsumer.getTokenSecret();
    }

	public static DefaultHttpClient getDefaultHttpClient() throws IndiceeAPIConnectorException {
		SchemeRegistry registry = new SchemeRegistry();

		registry.register(new Scheme("http", 80, PlainSocketFactory.getSocketFactory()));

		try {
			SSLSocketFactory sf = new SSLSocketFactory(SSLContext.getDefault(),
					SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);
			registry.register(new Scheme("https", 443, sf));
		} catch (NoSuchAlgorithmException e) {
			throw new IndiceeAPIConnectorException("Problem creating ssl connection", e);
		}

		ClientConnectionManager conman = new SingleClientConnManager(registry);
		return new DefaultHttpClient(conman);
	}
    
	private <R> R makeCallInternal(HttpUriRequest request, StreamProcessor<R> streamProcessor)
			throws IndiceeAPIConnectorException {
		// init a http client
		DefaultHttpClient httpClient = getDefaultHttpClient();
		try {
			signRequest(request);

			// send the data to get the response
			long start = System.currentTimeMillis();
			HttpResponse httpResponse;
			try {
				httpResponse = httpClient.execute(request);
			} catch (Exception e) {
				logger.error(e);
				throw new IndiceeAPIConnectorException(
						"Error sending HTTP request.", request.getURI()
								.getPath(), e);
			}

			int status = httpResponse.getStatusLine().getStatusCode();

			if (logger.isInfoEnabled()) {
				String consumerKey = commonsHttpConsumer.getConsumerKey();
				String token = commonsHttpConsumer.getToken();
				String method = request.getMethod();
				long took = System.currentTimeMillis() - start;
				URI uri = request.getURI();
				logger.info("INDICEE-API-REQUEST-COMPLETE|" + consumerKey + "|" + token + "|" + took + "|" + method + "|" + status + "|" + uri);
			}
			
			InputStream body;
			try {
				body = httpResponse.getEntity().getContent();
			} catch (Exception e) {
				logger.error(e);
				throw new IndiceeAPIConnectorException("Unexpected error getting response body.", request.getURI().getPath(), e);
			}
			// Checks to see if the call completed successfully
			if (status != 200) {
				try {
					String rtn = stringStreamProcessor.call(body);
					String path = request.getURI().getRawPath();
					throw createIndiceeAPIStatusException(status, rtn, path);
				} finally {
					try {
						if (body != null) {
							body.close();
						}
					} catch (IOException e) {
						throw new IndiceeAPIConnectorException(
								"Problem closing response stream.", e);
					}
				}
			}
			
			Header[] quotaHeader =httpResponse.getHeaders(REMAINING_QUOTA_HEADER);
			if(quotaHeader.length >0){
				try{
					remainingQuota = Integer.parseInt(quotaHeader[0].getValue());
				}catch(NumberFormatException e){
					logger.warn("Can not set remaining Quota - will set it to -1 (unknown)" , e);
					remainingQuota = -1;
				}
			}
			
			try {
				return streamProcessor.call(body);
			} finally {
				try {
					if (body != null) {
						body.close();
					}
				} catch (IOException e) {
					throw new IndiceeAPIConnectorException(
							"Problem closing response stream.", e);
				}
			}
		} finally {
			httpClient.getConnectionManager().shutdown();
		}
	}

    private IndiceeAPIStatusException createIndiceeAPIStatusException(int status, String message, String path) {
		int code = 0;
		try {
			Document document = XmlHelper.readXmlDocumentFromString(message);
			Element element = document.getDocumentElement();
			try {
				code = Integer.parseInt(XmlHelper.childText(element, "code"));
			} catch (NumberFormatException e) {

			}
			message = XmlHelper.childText(element, "message");
		} catch (Throwable t) {

		}
		return new IndiceeAPIStatusException(status, code, message, path);
	}

	/**
     * Makes a GET call to indicee.
     * 
     * @param path
     *            The url to connect to.
     * @param parameters
     *            The parameters to send. If null no parameters will be sent.
     * @return An Input stream of the response from the Indicee server.
     * @throws IndiceeAPIConnectorException
     */
    private <R> R makeGetCallInternal(String path, StreamProcessor<R> streamProcessor) throws IndiceeAPIConnectorException {
        HttpGet httpGet = new HttpGet(host + path);
        return makeCallInternal(httpGet, streamProcessor);
    }

    private <R> R makeDeleteCallInternal(String path, StreamProcessor<R> streamProcessor) throws IndiceeAPIConnectorException {
    	HttpDelete httpDelete = new HttpDelete(host + path);
        return makeCallInternal(httpDelete, streamProcessor);
    }
    
    /**
     * Makes a GET call to indicee.
     * 
     * @param path
     *            The url to connect to.
     * @param parameters
     *            The parameters to send. If null no parameters will be sent.
     * @return An Input stream of the response from the Indicee server.
     * @throws IndiceeAPIConnectorException
     */
    private <R> R makeGetCallInternal(String path, Map<String, ? extends Object> parameters, StreamProcessor<R> streamProcessor)
            throws IndiceeAPIConnectorException {
        StringBuilder queryParam = new StringBuilder();
        buildQueryParameters(parameters, queryParam);

        HttpGet httpGet = new HttpGet(host + path + queryParam);
        return makeCallInternal(httpGet, streamProcessor);
    }

	private void buildQueryParameters(Map<String, ? extends Object> parameters,
			StringBuilder queryParam) throws IndiceeAPIConnectorException {
		if (parameters != null && !parameters.isEmpty()) {
            queryParam.append("?");
            boolean firstEntry = true;
            for (Entry<String, ? extends Object> entry : parameters.entrySet()) {
            	if (firstEntry) {
            		firstEntry = false;
            	} else {
            		queryParam.append("&");
            	}
            	String key = entry.getKey();
            	Object value = entry.getValue();
            	queryParam.append(key);
            	queryParam.append("=");
            	if (value instanceof List) {
            		boolean firstItem = true;
            		for (Object item : (List<?>)value) {
            			if (firstItem) {
            				firstItem = false;
            			} else {
            				queryParam.append(",");
            			}
            			queryParam.append(IndiceeAPIService.urlEncode(String.valueOf(item)));
            		}
            	} else {
            		queryParam.append(IndiceeAPIService.urlEncode(String.valueOf(value)));
            	}
            }
        }
	}

    /**
     * Makes a GET call to the Indicee API, on the given path. This method will
     * return a parsed xml {@link Document}.
     * 
     * @param path
     *            The path to connect to.
     * @return A parsed xml {@link Document} of the server response.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there was an error connecting to the server, or if
     *             the server returned an error.
     */
    public Document makeGetCallRtnDocument(String path) throws IndiceeAPIConnectorException {
        return makeGetCallRtnDocument(path, null);
    }

    /**
     * Makes a GET call to the Indicee API, on the given path with the given
     * parameters. This method will return a parsed xml {@link Document}. <br/>
     * NOTE: Currently parameters are not being sent to indicee in get calls.
     * 
     * @param path
     *            The path to connect to.
     * @param parameters
     *            The parameters to send to the server. If null no parameters
     *            will be sent.
     * @return A parsed xml {@link Document} of the server response.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there was an error connecting to the server, or if
     *             the server returned an error.
     */
    public Document makeGetCallRtnDocument(String path, Map<String, ? extends Object> parameters)
            throws IndiceeAPIConnectorException {
    	return makeGetCallInternal(path, parameters, xmlStreamProcessor);
    }

    /**
     * Makes a GET call to the Indicee API, on the given path. This method will
     * return a String.
     * 
     * @param path
     *            The path to connect to.
     * @return A String of the server response.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there was an error connecting to the server, or if
     *             the server returned an error.
     */
    public String makeGetCallRtnString(String path) throws IndiceeAPIConnectorException {
        return makeGetCallInternal(path, stringStreamProcessor);
    }
    
    public String makeDeleteCallRtnString(String path) throws IndiceeAPIConnectorException {
        return makeDeleteCallInternal(path, stringStreamProcessor);
    }
    
    //added by aminy
    /**
     * Makes a GET call to the Indicee API, on the given path. This method will
     * return a Byte array.
     * 
     * @param path
     *            The path to connect to.
     * @return A String of the server response.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there was an error connecting to the server, or if
     *             the server returned an error.
     */
    public byte[] makeGetCallRtnByteArray(String path) throws IndiceeAPIConnectorException {
        return makeGetCallInternal(path, byteArrayStreamProcessor);
    }

    /**
	 * Makes a GET call to the Indicee API, on the given path. The result is
	 * written to the supplied output stream.
	 * 
	 * @param path
	 *            The path to connect to.
	 * @throws IndiceeAPIConnectorException
	 *             Thrown if there was an error connecting to the server, or if
	 *             the server returned an error.
	 */
    public void makeGetCallToStream(String path, OutputStream os) throws IndiceeAPIConnectorException {
        makeGetCallInternal(path, new OutputStreamProcessor(os));
    }

    /**
	 * Makes a GET call to the Indicee API, on the given path. The result is
	 * written to the supplied output stream.
	 * 
	 * @param path
	 *            The path to connect to.
	 * @param parameters
	 *            The parameters to send. If null no parameters will be sent.
	 * @param os
	 *            The output stream to write the data to           
	 * @throws IndiceeAPIConnectorException
	 *             Thrown if there was an error connecting to the server, or if
	 *             the server returned an error.
	 */
	public void makeGetCallToStream(String path, Map<String, ? extends Object> parameters, OutputStream os)
            throws IndiceeAPIConnectorException {
		makeGetCallInternal(path, parameters, new OutputStreamProcessor(os));
	}
    
    /**
     * Makes a GET call to the Indicee API, on the given path. This method will
     * return a String.
     * 
     * @param path
     *            The path to connect to.
     * @return A String of the server response.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there was an error connecting to the server, or if
     *             the server returned an error.
     */
    public String makeGetCallRtnString(String path, Map<String, ? extends Object> parameters)
            throws IndiceeAPIConnectorException {
        return makeGetCallInternal(path, parameters, stringStreamProcessor);
    }

    /**
     * Makes a POST call to indicee.
     * 
     * @param path
     *            The url to connect to.
     * @param parameters
     *            The parameters to send. If null no parameters will be sent.
     * @return An Input stream of the response from the Indicee server.
     * @throws IndiceeAPIConnectorException
     */
    private <R> R makePostCallInternal(String path, Map<String, ? extends Object> parameters, StreamProcessor<R> streamProcessor)
            throws IndiceeAPIConnectorException {
        HttpPost httpPost = new HttpPost(host + path);
        if (parameters != null && !parameters.isEmpty()) {

            if (parametersContainsFileOrInputStream(parameters)) {

                // build post data. File upload needs multipart form data.
                // see http://www.w3.org/TR/html401/interact/forms.html#h-17.13.4.2
                MultipartEntity mentity = new MultipartEntity();

                for (String key : parameters.keySet()) {
                    Object value = parameters.get(key);
                    if (value instanceof File) {
                        ContentBody fbody;
                        try {
                            fbody = new InputStreamBody(new FileInputStream((File) value), ((File) value).getName());
                        } catch (FileNotFoundException e) {
                        	logger.error(e);
                            throw new IndiceeAPIConnectorException("Error sending file: " + ((File) value).getName()
                                    + " could not find file on local machine.", path, e);
                        }
                        mentity.addPart(key, fbody);
                    } else if (value instanceof InputStreamWrapper) {
                        ContentBody fbody;
                        InputStreamWrapper streamWrapper = (InputStreamWrapper) value;
                        MimeType mimeType = streamWrapper.getMimeType();
                        if (mimeType != null) {
                        	StringBuilder contentType = new StringBuilder(mimeType.getContentType());
                        	String charset = streamWrapper.getCharset();
                        	if (charset != null) {
                        		contentType.append("; charset=");
                        		contentType.append(charset);
                        	}
                        	fbody = new InputStreamBody(streamWrapper.getInputSteam(), contentType.toString(), streamWrapper.getFileName());
                        } else {
                            fbody = new InputStreamBody(streamWrapper.getInputSteam(), streamWrapper.getFileName());
                        }
                        mentity.addPart(key, fbody);
                    } else {
                        try {
                            mentity.addPart(key, new StringBody(parameters.get(key).toString()));
                        } catch (UnsupportedEncodingException e) {
                        	logger.error(e);
                            throw new IndiceeAPIConnectorException("Unexpected error encoding string.", path, e);
                        }
                    }
                }
                httpPost.setEntity(mentity);
            } else {
                try {
                	StringBuilder sb = new StringBuilder();
                	for (Map.Entry<String, ? extends Object> entry : parameters.entrySet()) {
                		if (sb.length() > 0) {
                			sb.append("&");
                		}
                		String key = entry.getKey();
                		Object value = entry.getValue();
                		if (logger.isTraceEnabled()) {
                			logger.trace("key: " + key + " value: " + value);
                		}
                   		String valueString = value.toString();
                   		String escapedValue = URLEncoder.encode(valueString, "UTF-8");
                		sb.append(key);
                   		sb.append("=");
               			sb.append(escapedValue);
                	}
                	StringEntity entity = new StringEntity(sb.toString(), MimeType.FORM.getContentType(), "utf-8");
                    httpPost.setEntity(entity);
                } catch (UnsupportedEncodingException e) {
                	logger.error(e);
                    throw new IndiceeAPIConnectorException("Unexpected error encoding string.", path, e);
                }
            }
        }

        return makeCallInternal(httpPost, streamProcessor);
    }

    private <R> R makePostCallInternal(String path, String body, StreamProcessor<R> streamProcessor, MimeType mimeType) throws IndiceeAPIConnectorException {
        HttpPost httpPost = new HttpPost(host + path);
        if (body != null && !body.isEmpty()) {
            StringEntity entity;
            try {
                entity = new StringEntity(body, mimeType.getContentType(), "utf-8");
            } catch (UnsupportedEncodingException e) {
            	logger.error(e);
                throw new IndiceeAPIConnectorException("Unexpected error encoding string.", path, e);
            }
            
            httpPost.setEntity(entity);
        }

        return makeCallInternal(httpPost, streamProcessor);
    }

    /**
     * Makes a POST call to the Indicee API, on the given path. This method will
     * return a parsed xml {@link Document}.
     * 
     * @param path
     *            The path to connect to.
     * @return A parsed xml {@link Document} of the server response.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there was an error connecting to the server, or if
     *             the server returned an error.
     */
    public Document makePostCallRtnDocument(String path) throws IndiceeAPIConnectorException {
        return makePostCallRtnDocument(path, "");
    }

    /**
     * Makes a POST call to the Indicee API, on the given path with the given
     * parameters. This method will return a parsed xml {@link Document}. The
     * the parameters map contains a {@link File} the file will be sent to
     * indicee. If it contains an {@link InputStreamWrapper} the input stream
     * will be send to indicee.
     * 
     * @param path
     *            The path to connect to.
     * @param parameters
     *            The parameters to send to the server. If null no parameters
     *            will be sent. If this contains a {@link File} or
     *            {@link InputStreamWrapper} it will be properly sent to
     *            Indicee.
     * @return A parsed xml {@link Document} of the server response.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there was an error connecting to the server, or if
     *             the server returned an error.
     */
    public Document makePostCallRtnDocument(String path, Map<String, ? extends Object> parameters)
            throws IndiceeAPIConnectorException {
        return makePostCallInternal(path, parameters, xmlStreamProcessor);
	}

    public Document makePostCallRtnDocument(String path, String string) throws IndiceeAPIConnectorException {
        return makePostCallInternal(path, string, xmlStreamProcessor, MimeType.XML);
	}

    /**
     * Makes a POST call to the Indicee API, on the given path. This method will
     * return a String.
     * 
     * @param path
     *            The path to connect to.
     * @return A String of the server response.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there was an error connecting to the server, or if
     *             the server returned an error.
     */
    public String makePostCallRtnString(String path) throws IndiceeAPIConnectorException {
        return makePostCallRtnString(path, "");
    }

    /**
     * Makes a POST call to the Indicee API, on the given path with the given
     * parameters. This method will return a String. The the parameters map
     * contains a {@link File} the file will be sent to indicee. If it contains
     * an {@link InputStreamWrapper} the input stream will be send to indicee.
     * 
     * @param path
     *            The path to connect to.
     * @param parameters
     *            The parameters to send to the server. If null no parameters
     *            will be sent. If this contains a {@link File} or
     *            {@link InputStreamWrapper} it will be properly sent to
     *            Indicee.
     * @return A String of the server response.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there was an error connecting to the server, or if
     *             the server returned an error.
     */
    public String makePostCallRtnString(String path, Map<String, ? extends Object> parameters)
            throws IndiceeAPIConnectorException {
        return makePostCallInternal(path, parameters, stringStreamProcessor);
    }

    /**
     * Makes a POST call to the Indicee API, on the given path with the given
     * body. This method will return a String.
     * 
     * @param path
     *            The path to connect to.
     * @param body
     *            The body string to send to the server. If null no body will be
     *            sent.
     * @return A String of the server response.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there was an error connecting to the server, or if
     *             the server returned an error.
     */
    public String makePostCallRtnString(String path, String body) throws IndiceeAPIConnectorException {
        return makePostCallInternal(path, body, stringStreamProcessor, MimeType.XML);
    }

    /**
     * Makes a POST call to the Indicee API, on the given path with the given
     * body. This method will return a String.
     * 
     * @param path
     *            The path to connect to.
     * @param body
     *            The body string to send to the server. If null no body will be
     *            sent.
     * @param contentType
     *            The content type for the body data           
     * @return A String of the server response.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there was an error connecting to the server, or if
     *             the server returned an error.
     */
    public String makePostCallRtnString(String path, String body, MimeType mimeType) throws IndiceeAPIConnectorException {
        return makePostCallInternal(path, body, stringStreamProcessor, mimeType);
    }

    private boolean parametersContainsFileOrInputStream(Map<String, ? extends Object> parameters) {
        for (Object value : parameters.values()) {
            if (value instanceof File || value instanceof InputStreamWrapper) {
                return true;
            }
        }
        return false;
    }

    /**
     * This is needed because the commonsHttpConsumer is not thread safe.
     * 
     * @param request
     * @throws IndiceeAPIConnectorException
     */
    private synchronized void signRequest(HttpUriRequest request) throws IndiceeAPIConnectorException {
        try {
            // authenticate the request with oauth
            commonsHttpConsumer.sign(request);
        } catch (Exception e) {
        	logger.error(e);
            throw new IndiceeAPIConnectorException("Error signing HTTP request with OAuth.",
                    request.getURI().getPath(), e);
        }
    }

    @Override
    public String toString() {
        return "Host: " + host + "\nConsumer Key: " + commonsHttpConsumer.getConsumerKey()
                + "\nToken Key: " + commonsHttpConsumer.getToken() + "\nRemaining Quota: " + remainingQuota;
    }
    
    /**
     * will return remaining call quota - if -1 => unknown
     * @return
     */
    public int getRemainingQuota() {
		return remainingQuota;
	}
}