package com.indicee.api.connector;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.connector.exception.IndiceeAPIConnectorFactoryException;
import com.indicee.api.connector.exception.IndiceeAPIConnectorFactoryException.Problem;
import com.indicee.api.objects.auth.Token;
import com.indicee.api.services.impl.AuthServiceImpl;
import com.indicee.api.services.interfaces.AuthService;
import com.indicee.api.util.XmlHelper;

public class IndiceeAPIConnectorFactory {

	private final String defaultHost;

	private final String appKey;
	private final String appSecret;
	private final String tokenKey;
	private final String tokenSecret;
	
	public IndiceeAPIConnectorFactory(String defaultHost, String appKey,
			String appSecret, String tokenKey,
			String tokenSecret) {
		super();
		this.defaultHost = defaultHost;
		this.appKey = appKey;
		this.appSecret = appSecret;
		this.tokenKey = tokenKey;
		this.tokenSecret = tokenSecret;
	}
	
	public IndiceeAPIConnectorFactory(Properties properties) {
		super();
		this.defaultHost = properties.getProperty("indicee.host");
		this.appKey = properties.getProperty("app.key");
		this.appSecret = properties.getProperty("app.secret");
		this.tokenKey = properties.getProperty("token.key");
		this.tokenSecret = properties.getProperty("token.secret");
	}
	
	public IndiceeAPIConnectorFactory(Element connectorElement){
		super();
		this.defaultHost = XmlHelper.childText(connectorElement, "host");
		this.appKey = XmlHelper.childText(connectorElement, "appkey");
		this.appSecret = XmlHelper.childText(connectorElement, "appsecret");
		this.tokenKey = XmlHelper.childText(connectorElement, "tokenkey");
		this.tokenSecret = XmlHelper.childText(connectorElement, "tokensecret");
	}
	
	
	public String getDefaultHost() {
		return defaultHost;
	}
	
	
	public String getappKey() {
		return appKey;
	}

	public String getappSecret() {
		return appSecret;
	}

	public IndiceeAPIConnector createUserConnector(String token, String secret, String host) {
		return createConnector(host, token, secret);
	}
	
	public IndiceeAPIConnector createUserConnector(String key, String host)  throws IndiceeAPIConnectorFactoryException {
		if (host == null || host.length() == 0) {
			host = defaultHost;
		}
		if (key == null || key.length() == 0) {
			throw new IndiceeAPIConnectorFactoryException(Problem.NoKey);
		}
		IndiceeAPIConnector applicationConnector = createApplicationConnector(host);
		Long user = checkKey(applicationConnector, key);
		if (user == null) {
			throw new IndiceeAPIConnectorFactoryException(Problem.InvalidKey);
		}

		AuthService authService = new AuthServiceImpl(applicationConnector);
		Token token = getToken(authService, user);
		if (token == null) {
			throw new IndiceeAPIConnectorFactoryException(Problem.NotAuthorized);
		}

		IndiceeAPIConnector userConnector = createConnector(host,
				token.getKey(), token.getSecret());
		return userConnector;
	}
	
	public IndiceeAPIConnector createApplicationConnector(String host) {
		IndiceeAPIConnector applicationConnector = createConnector(host, tokenKey, tokenSecret);
		return applicationConnector;
	}
	
	private IndiceeAPIConnector createConnector(String host, String tokenKey,
			String tokenSecret) {
		if (host == null || host.length() == 0) {
			host = defaultHost;
		}
		return new IndiceeAPIConnector(host, appKey, appSecret, tokenKey, tokenSecret);
	}

	private Long checkKey(IndiceeAPIConnector applicationConnector, String token) throws IndiceeAPIConnectorFactoryException {
		Map<String, String> params = new HashMap<String, String>(1);
		params.put("auth_key", token);
		try {
			Document document = applicationConnector.makePostCallRtnDocument("/api/dynauth/checkkey", params);
			Element element = document.getDocumentElement();
			String id = XmlHelper.childText(element, "id");
			if (id != null && id.length() > 0) {
				try {
					return Long.parseLong(id);
				} catch (NumberFormatException e) {
				}
			}
		} catch (IndiceeAPIConnectorException e) {
			throw new IndiceeAPIConnectorFactoryException(Problem.ErrorCheckingKey, e);
		}
		return null;
	}

	private static Token getToken(AuthService authService, Long userID) throws IndiceeAPIConnectorFactoryException {
		if (userID != null) {
			try {
				List<Token> tokens = authService.listTokens();
				for (Token token : tokens) {
					Long id = token.getOwnerID();
					if (userID.equals(id)) {
						return token;
					}
				}
			} catch (IndiceeAPIConnectorException e) {
				throw new IndiceeAPIConnectorFactoryException(Problem.ErrorListingTokens, e);
			}
		}
		return null;
	}
}
