package com.indicee.api.objects;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.indicee.api.exceptions.QualifiedNameSyntaxException;


/** This class is a copy of com.indicee.model.framework.shared.model.QualifiedName
 *  to be used on the application side
 * 
 * An Indicee Qualified Name (IQN) is a URI-like object that uniquely identifies
 * a persistent object in a global namespace. 
 * 
 * An example of a valid IQN is:
 * 
 * "iqn.1://indicee.com/1/dataModel/dataSets/MyDataSet"
 * 
 * Currently we only support the scheme iqn.1, which indicates a version 1 IQN.
 * The second part of the IQN is the hostname, followed by the ID of the user.
 * The userId is a required part of the path and must be followed by at least
 * one path segment.
 * 
 * Note that valid IQNs must have their path segments percent encoded. Be
 * careful when using the qualified name API because some methods take a string
 * argument that is assumed to be escaped using percent-encoding and some assume
 * the argument is not escaped.
 * 
 * @author Doug Janzen
 */
public class QualifiedName implements Serializable, Comparable<QualifiedName> {

    private static final long serialVersionUID = 9076630281762934742L;

    public static final String reservedChars = " %:/?#[]@!$&'()*+,;=";
    public static final String unreservedChars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-._~";

    public static final QualifiedName UNNAMED = new QualifiedName("iqn.2://indicee.com/0/0/Unnamed");

    private String normalized;

    private transient String scheme;
    private transient int schemeVersion;
    private transient String hostName;
    private transient long userId;
    private transient ArrayList<String> pathSegments;
    private transient long spaceId;

    /** No-arg constructor for serialization. */
    protected QualifiedName() {
    }

    private static QualifiedName copy(QualifiedName target) {
        QualifiedName qn = new QualifiedName();
        qn.copyThis(target);
        return qn;
    }
    
    public static boolean isBlank(String value) {
        return value == null || value.trim().length() == 0;
    }
    
    private void copyThis(QualifiedName target) {
        this.normalized = target.normalized;
        this.scheme = target.scheme;
        this.schemeVersion = target.schemeVersion;
        this.hostName = target.hostName;
        this.pathSegments = new ArrayList<String>(target.pathSegments);
        this.userId = target.userId;
        this.spaceId = target.spaceId;
    }

    /**
     * Percent encodes the given string.
     */
    public static final String escape(String value) {

        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < value.length(); i++) {
            char thisChar = value.charAt(i);
            if (thisChar == '/') {
                buf.append("%" + toHexString('-'));
            } else if (thisChar == '\\') {
                buf.append("%" + toHexString('_'));
            } else if (unreservedChars.indexOf(thisChar) >= 0) {
                buf.append(thisChar);
            } else {
                buf.append("%" + toHexString(thisChar));
            }
        }
        return buf.toString();
    }

    private void ensureInit() {
        if (scheme != null)
            return;

        // Check that the colon is in the right place
        int colonIdx = normalized.indexOf(':');

        scheme = normalized.substring(0, colonIdx);
        schemeVersion = 2;

        String[] rest = normalized.substring(colonIdx + 3, normalized.length()).split("/");
        hostName = rest[0];

        userId = Long.parseLong(rest[1]);
        spaceId = Long.parseLong(rest[2]);

        pathSegments = new ArrayList<String>(rest.length - 1);
        for (int i = 1; i < rest.length; i++)
            pathSegments.add(rest[i]);
    }

    private static String toHexString(char c) {
        String res = Integer.toHexString(c).toUpperCase();

        switch (res.length()) {
        case 1:
            return "0" + res;
        case 2:
            return res;
        case 3:
            return "u0" + res;
        default:
            return "u" + res;
        }
    }

    /**
     * Replaces percent encoded characters in a string with the original
     * characters.
     */
    public static final String unescape(String value) {

        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < value.length(); i++) {
            char thisChar = value.charAt(i);
            if (thisChar == '%') {
                if (i + 3 > value.length()) {
                    throw new IllegalArgumentException(
                            "Cannot unescape string with illegal percent encoding at character " + i + ": " + value);
                }

                String hex = value.substring(i + 1, i + 3);

                if (hex.charAt(0) == 'u') {
                    if (i + 6 > value.length()) {
                        throw new IllegalArgumentException(
                                "Cannot unescape string with illegal 16bit percent encoding at character " + i + ": "
                                        + value);
                    }
                    hex = value.substring(i + 2, i + 6);
                    i += 5;
                } else {
                    i += 2;
                }

                int intVal = Integer.parseInt(hex, 16);
                char unescapedChar = (char) intVal;
                if (unescapedChar == '-') {
                    unescapedChar = '/';
                } else if (unescapedChar == '_') {
                    unescapedChar = '\\';
                }
                buf.append(unescapedChar);

            } else if (unreservedChars.indexOf(thisChar) >= 0) {
                buf.append(thisChar);
            } else {
                throw new IllegalArgumentException("Cannot unescape string with illegal character '" + thisChar
                        + "' in: " + value);
            }
        }
        return buf.toString();
    }

    /**
     * Creates a base qualifiedname
     * 
     * @param userId
     * @param spaceId
     * @return
     */
    public static QualifiedName make(long userId, long spaceId) {
        return new QualifiedName("iqn.2://indicee.com/" + userId + "/" + spaceId);
    }

    /**
     * Creates a copy of this qualified name in the given space.
     * 
     * @param spaceId
     * @return
     */
    public QualifiedName moveToSpaceId(long spaceId) {
        ensureInit();
        QualifiedName res = copy(this);
        res.pathSegments.set(1, Long.toString(spaceId));
        res.spaceId = spaceId;
        res.normalize();
        return res;
    }

    /**
     * Moves a qualified name to the specified user and space.
     * 
     * @param user
     * @param spaceId
     * @return
     */
    public QualifiedName moveToUserSpaceId(long user, long spaceId) {
        ensureInit();
        QualifiedName res = copy(this);
        res.pathSegments.set(0, Long.toString(user));
        res.pathSegments.set(1, Long.toString(spaceId));
        res.spaceId = spaceId;
        res.userId = user;
        res.normalize();
        return res;
    }

    /**
     * Creates a qualified name from a base name and an unescaped string
     * representing a path.
     */
    public static QualifiedName make(QualifiedName base, String unescapedString) {

        String[] segments = unescapedString.split("/");
        List<String> escaped = new ArrayList<String>();

        for (String segment : segments) {
            escaped.add(escape(segment));
        }

        String path = join(escaped, true);

        return new QualifiedName(base, path);

    }

    /**
     * Returns true if the given path segment is a valid, unescaped string.
     */
    private boolean isValidPathSegment(String segment) {
        try {
            unescape(segment);
            return true;
        } catch (IllegalArgumentException e) {
            return false;
        }
    }

    /**
     * Creates a qualified name from the given string. The string must be of a
     * valid form and have all reserved characters percent encoded. An example
     * of a valid IQN is:
     * 
     * <code>
     * iqn.1://indicee.com/1/dataModel/dataSets/MyDataSet
     * </code>
     * 
     */
    public QualifiedName(String qualifiedName) throws QualifiedNameSyntaxException {

        if (qualifiedName == null) {
            throw new NullPointerException("Cannot make a qualified name with a null value.");
        }

        String qn = qualifiedName; // renaming the value to be parsed just to make the code more concise.

        //at one point in the past we changed the encoding of / to avoid them being rewritten by apache
        //we need to support this in old xml in which they may still exist.
        qn = qn.replaceAll("%2F", "%2D");

        // Support existing qualified names for now, converting them to IQNs.
        if (qn != null && qn.startsWith("indicee/")) {
            qn = "iqn.1://indicee.com" + qn.substring(qn.indexOf('/'));
        }

        // Check that given qualified name is not blank and is at least long enough to form the
        // shortest possible qualified name.
        if (isBlank(qualifiedName) || qualifiedName.length() < "iqn.1://a/b".length()) {
            throw new QualifiedNameSyntaxException("Invalid qualified name: " + qn);
        }

        // Check that the colon is in the right place
        int colonIdx = qn.indexOf(':');
        if (colonIdx != 5) {
            throw new QualifiedNameSyntaxException("Invalid qualified name: " + qn);
        }

        scheme = qn.substring(0, colonIdx);

        if (scheme.equals("iqn.2")) {
            parseScheme2(qn, colonIdx);
        } else if (scheme.equals("iqn.1")) {
            parseScheme1(qn, colonIdx);
        } else
            throw new QualifiedNameSyntaxException("Invalid scheme: " + scheme);

    }

    private void parseScheme2(String qn, int colonIdx) {
        // Currently we only support version 1.
        schemeVersion = 2;
        scheme = "iqn.2";

        // Make sure the part after the colon indicates the authority.
        String rest = qn.substring(colonIdx + 1);

        if (isBlank(rest) || !rest.startsWith("//")) {
            throw new QualifiedNameSyntaxException("Invalid qualified name: " + qn);
        }

        // Get the hostname
        int pathRootIdx = rest.indexOf('/', 2);

        // Make sure we have a hostname and at least one path segment
        if (pathRootIdx < 2) {
            throw new QualifiedNameSyntaxException("Invalid qualified name: " + qn);
        }

        hostName = rest.substring(2, pathRootIdx);

        // Check that we have a valid hostname.
        if (isBlank(hostName) || !hostName.matches("^[a-zA-Z0-9\\.]*$")) {
            throw new QualifiedNameSyntaxException("Invalid qualified name: " + qn);
        }

        // Get the path
        String path = rest.substring(pathRootIdx + 1);

        pathSegments = new ArrayList<String>();

        evaluateRelativePath(path);
        normalize();
    }

    private void parseScheme1(String qn, int colonIdx) {

        String updated = qn.replaceAll("iqn\\.1://indicee.com/([0-9]+)/", "iqn\\.2://indicee.com/$1/0/");
        parseScheme2(updated, colonIdx);

    }

    /**
     * Creates a qualified name from a base namespace and a relative path. The
     * path must be percent-encoded.
     */
    public QualifiedName(QualifiedName namespace, String path) throws QualifiedNameSyntaxException {

        this.scheme = namespace.scheme;
        this.schemeVersion = namespace.schemeVersion;
        this.hostName = namespace.hostName;
        this.pathSegments = new ArrayList<String>();

        if (isBlank(path)) {
            this.pathSegments.addAll(namespace.pathSegments);
        } else if (path.startsWith("/")) {
            if (path.length() < 2) {
                throw new QualifiedNameSyntaxException("Invalid path: " + path);
            }
            evaluateRelativePath(path.substring(1));
        } else {
            this.pathSegments.addAll(namespace.pathSegments);
            evaluateRelativePath(path);
        }
        normalize();

    }

    /**
     * Evaluates the give relative path given an already initialized
     * pathSegments.
     */
    private void evaluateRelativePath(String path) {

        // We don't allow paths ending with a slash.
        if (path.trim().endsWith("/")) {
            throw new QualifiedNameSyntaxException("Invalid path: " + path);
        }

        // Split the path into slash separated segments
        String[] segments = path.split("/");

        for (String segment : segments) {

            if (isBlank(segment)) {
                throw new QualifiedNameSyntaxException("Invalid path: " + path);
            }

            if (segment.equals(".")) {
                continue;
            } else if (segment.equals("..")) {

                if (pathSegments.size() < 1) {
                    throw new QualifiedNameSyntaxException("Invalid path: " + path);
                }
                pathSegments.remove(pathSegments.size() - 1);
            } else {

                if (!isValidPathSegment(segment)) {
                    throw new QualifiedNameSyntaxException("Invalid path segment: " + segment);
                }
                pathSegments.add(segment);
            }
        }

        if (pathSegments.size() < 2) {
            throw new QualifiedNameSyntaxException("Not enough segments in path: " + path);
        }

        String userIdSegment = pathSegments.get(0);

        try {
            userId = Long.parseLong(userIdSegment);
        } catch (NumberFormatException e) {
            throw new QualifiedNameSyntaxException("Invalid user ID segment: " + path);
        }

        String spaceIdSegment = pathSegments.get(1);

        try {
            spaceId = Long.parseLong(spaceIdSegment);
        } catch (NumberFormatException e) {
            throw new QualifiedNameSyntaxException("Invalid space ID segment: " + path);
        }

    }

    /**
     * Initializes the normalized field.
     */
    private void normalize() {
        normalized = toString(true);
    }

    /**
     * Determines if the provided value is a valid string representation of a
     * qualified name.
     */
    public static boolean isValid(String value) {
        try {
            new QualifiedName(value);
            return true;
        } catch (QualifiedNameSyntaxException e) {
            return false;
        }
    }

    /**
     * Creates a qualified name by appending the given path segment to this
     * qualified name. The given segment will be percent-encoded.
     */
    public QualifiedName append(String segment) {
        ensureInit();
        return new QualifiedName(this, escape(segment));
    }

    /**
     * Returns a qualified name representing the parent of this qualified name.
     */
    public QualifiedName parent() {
        ensureInit();
        if (pathSegments.isEmpty()) {
            throw new IllegalStateException("The qualified name is empty");
        }
        return new QualifiedName(this, "..");
    }

    /**
     * Returns the path partion of the qualified name as a percent-encode
     * string, delimited by "/"s.
     */
    public String getPath() {
        ensureInit();
        return join(pathSegments, true);
    }

    /**
     * Returns a qualified name that contains the prefix of the given size.
     * 
     * @param size
     *            The size of the prefix to return (not the zero-based index of
     *            the last part!)
     */
    public QualifiedName prefix(int size) {
        ensureInit();
        if (pathSegments.size() < size) {
            throw new IllegalArgumentException(
                    "Cannot get prefix for qualified name because the qualified name is shorter than the requested prefix size. Requested size="
                            + size + ", QualifiedName=" + this);
        }
        QualifiedName newQn = copy(this);
        newQn.pathSegments.clear();
        newQn.pathSegments.addAll(pathSegments.subList(0, size));
        return newQn;
    }

    /**
     * Returns the last segment of the path as an unescaped string.
     */
    public String getName() {
        ensureInit();
        return getName(false);
    }

    /**
     * Returns the last segment of the path with optional escaping.
     */
    public String getName(boolean escaped) {
        ensureInit();
        if (pathSegments.isEmpty()) {
            throw new IllegalStateException("The qualified name is empty");
        }
        if (escaped) {
            return pathSegments.get(pathSegments.size() - 1);
        } else {
            return unescape(pathSegments.get(pathSegments.size() - 1));
        }
    }

    /**
     * Returns the normalized string representing this qualified name. The
     * normalized string has all relative parts evaluated and retains all
     * percent-encoding.
     * 
     * Returning an unescaped string here would make for nicer reading of log
     * files, etc. but the normalized string is better for putting in XML and is
     * more friendly to scripts that want to parse log files or write an IQN out
     * to a string and then recreate an IQN back from that string.
     */
    @Override
    public String toString() {
        return normalized;
    }

    /**
     * Returns the qualified name as a string, with percent-encoding optional.
     */
    public String toString(boolean escaped) {
        ensureInit();
        return scheme + "://" + hostName + "/" + join(pathSegments, escaped);
    }

    /**
     * Wrapper getter method to be used by request factory proxies
     * 
     * @return
     */
    public String getUnescaped() {
        ensureInit();
        return toString(false);
    }

    /**
     * Wrapper getter method to be used by request factory proxies
     * 
     * @return
     */
    public String getEscaped() {
        ensureInit();
        return toString(true);
    }

    /**
     * Joins the given list of strings. The segments can be optionally escaped
     * or not.
     */
    private static String join(List<String> bits, boolean escaped) {
        StringBuilder sb = new StringBuilder();
        boolean first = true;
        for (String part : bits) {
            if (first) {
                first = false;
            } else {
                sb.append("/");
            }
            if (escaped) {
                sb.append(part);
            } else {
                sb.append(unescape(part));
            }
        }

        return sb.toString();
    }

    /**
     * This function moves names between namespaces. e.g. if we move names from
     * namespace "iqn.1://indicee.com/1/animal" to
     * "iqn.1://indicee.com/1/mammal" then
     * 
     * <code>
     * iqn.1://indicee.com/1/animal/cat -> iqn.1://indicee.com/1/mammal/cat
     * </code>
     * <br/>
     * <code>
     * iqn.1://indicee.com/1/animal/small/dog -> iqn.1://indicee.com/1/mammal/small/dog 
     * </code>
     * <br/>
     * <code>
     * iqn.1://indicee.com/1/car/civic -> iqn.1://indicee.com/1/car/civic
     * </code>
     * 
     * @param oldNamespace
     * @param newNamespace
     * @return
     */
    public QualifiedName move(QualifiedName oldNamespace, QualifiedName newNamespace) {
        ensureInit();
        if (this.pathSegments.size() > oldNamespace.pathSegments.size()
                && pathSegments.subList(0, oldNamespace.pathSegments.size()).equals(oldNamespace.pathSegments)) {

            pathSegments.subList(oldNamespace.pathSegments.size(), pathSegments.size());

            ArrayList<String> newParts = new ArrayList<String>(newNamespace.pathSegments);
            newParts.addAll(pathSegments.subList(oldNamespace.pathSegments.size(), pathSegments.size()));

            QualifiedName qn = copy(this);
            qn.pathSegments = newParts;
            qn.normalize();
            return qn;
        }
        return this;
    }

    /**
     * 
     * True iff the parameter is a name within the namespace defined by this
     * name.
     * 
     * @param name
     * @return true iff the parameter is a descendant of this namespace
     */
    public boolean isDescendant(QualifiedName name) {
        ensureInit();
        if (name.pathSegments.size() <= this.pathSegments.size())
            return false;

        return this.pathSegments.equals(name.pathSegments.subList(0, this.pathSegments.size()));
    }

    @Override
    public int compareTo(QualifiedName o) {
        return this.normalized.compareTo(o.normalized);
    }

    public String getHostName() {
        ensureInit();
        return hostName;
    }

    public String getScheme() {
        ensureInit();
        return scheme;
    }

    public int getSchemeVersion() {
        ensureInit();
        return schemeVersion;
    }

    /**
     * Returns the path portion of the qualified name as a list. The segments
     * will be percent-encoded and will include the userId segment of the path.
     */
    public List<String> getPathSegments() {
        ensureInit();
        return Collections.unmodifiableList(pathSegments);
    }

    public String getNormalized() {
        return normalized;
    }

    /**
     * Returns the path segments as a string. The returned path will be percent
     * encoded if "escaped" is true.
     */
    public String getPath(boolean escaped) {
        ensureInit();
        return join(pathSegments, escaped);
    }

    public long getUserId() {
        ensureInit();
        return userId;
    }

    public long getSpaceId() {
        ensureInit();
        return spaceId;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((normalized == null) ? 0 : normalized.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        QualifiedName other = (QualifiedName) obj;
        if (normalized == null) {
            if (other.normalized != null)
                return false;
        } else if (!normalized.equals(other.normalized))
            return false;
        return true;
    }

}
