package com.indicee.api.objects;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * This is a view window to be used when using the api to return reports data.
 * Added ability to deal with less or more than 2 dimensions. Tried to keep it compatible with two dimension 
 * @author aminy
 * 
 */
public class ViewWindow implements Serializable {

	private static final long serialVersionUID = 3231037015409098097L;
    private static final int DEFAULT_PAGE_SIZE = 100;
	public static final int MaxSpan = Integer.MAX_VALUE;
	
    List<Integer> startCoordinates = new ArrayList<Integer>();
    List<Integer> extents = new ArrayList<Integer>();
    
    
    /**
     * Creates a default view window that returns the first 100 rows and 100
     * columns
     */
    public ViewWindow() {
        this(0, DEFAULT_PAGE_SIZE);
    }

    /**
     * Creates a view window with the specific row start and num rows.
     * 
     * @param rowStart
     *            The row to start the window at.
     * @param numRows
     *            The number of rows to return.
     */
    public ViewWindow(int rowStart, int numRows) {
        this(rowStart, numRows, 0, DEFAULT_PAGE_SIZE);
    }

    /**
     * Creates a view window with all bounds specified.
     * 
     * @param rowStart
     *            The row to start the window at.
     * @param numRows
     *            The number of rows to return.
     * @param columnStart
     *            The column to start the window at.
     * @param numColumns
     *            The number of columns to return.
     */
    public ViewWindow(int rowStart, int numRows, int columnStart, int numColumns) {
    	startCoordinates.add(rowStart);
    	startCoordinates.add(columnStart);
    	
    	extents.add(numRows);
    	extents.add(numColumns);
    }
    
    public ViewWindow(List<Integer> startCoordinates, List<Integer> extents){
    	if(startCoordinates.size()!=extents.size()){
    		//ERROR!
    	}
    	
    	this.startCoordinates = startCoordinates;
    	this.extents = extents;
    }
    
    /**
     * Create default view window for the specified number of axes.
     * So it'll be at origin and then an extent of 100 for the first 2 axes and then an extent of 1 for subsequent axes
     * @param nAxes
     */
    public ViewWindow(int nAxes) {
    	for (int i=0;i<nAxes;++i) {
    		startCoordinates.add(0);
        	extents.add(DEFAULT_PAGE_SIZE);
    	}
    }
    
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((extents == null) ? 0 : extents.hashCode());
		result = prime
				* result
				+ ((startCoordinates == null) ? 0 : startCoordinates.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ViewWindow other = (ViewWindow) obj;
		if (extents == null) {
			if (other.extents != null)
				return false;
		} else if (!extents.equals(other.extents))
			return false;
		if (startCoordinates == null) {
			if (other.startCoordinates != null)
				return false;
		} else if (!startCoordinates.equals(other.startCoordinates))
			return false;
		return true;
	}

	/**
     * Returns a string representation of the view window. This is the actual
     * query parameter.
     */
    @Override
    public String toString() {
    	StringBuilder sb = new StringBuilder();
    	
    	for(Integer i:startCoordinates){
    		if(sb.length()==0){
    			sb.append(i);
    		}else{
    			sb.append(",").append(i);
    		}
    	}
    	
    	for(Integer i:extents){
    		if(sb.length()==0){
    			sb.append(i);
    		}else{
    			sb.append(",").append(i);
    		}
    	}
    	
        return sb.toString();
    }
}
