package com.indicee.api.parser;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.joda.time.format.DateTimeFormat; 
import org.joda.time.format.DateTimeFormatter;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.indicee.api.util.XmlHelper;

public abstract class IndiceeParser {

	public static abstract class NodeParser<T> {
		private final String tag;
		public NodeParser(String tag) {
			if (tag == null) {
				throw new IllegalArgumentException();
			}
			this.tag = tag;
		}
		
		public abstract T parse(Node node);
		
		public final T parse(Document document) {
			Element element = document.getDocumentElement();
			return parse(element);
		}
		
		public final List<T> parseList(Node parent) {
			NodeList childNodes = parent.getChildNodes();
			int nChildNodes = childNodes.getLength();
			List<T> list = new ArrayList<T>(nChildNodes);
			for (int i=0;i<nChildNodes;++i) {
				Node childNode = childNodes.item(i);
				String nodeName = childNode.getNodeName();
				if (tag.equals(nodeName)) {
					T t = parse(childNode);
					list.add(t);
				}
			}
			return list;
		}

		public final List<T> parseList(Document document) {
			Element element = document.getDocumentElement();
			return parseList(element);
		}
	}
	
	
    private static final DateTimeFormatter dateFormats[] = new DateTimeFormatter[] {
            DateTimeFormat.forPattern("yyyy-MM-dd'T'HH:mm:ss'Z'Z"),
            DateTimeFormat.forPattern("yyyy-MM-dd'T'HH:mm:ss'Z'"),
            DateTimeFormat.forPattern("yyyy-MM-dd'T'HH:mm:ssZZ"),
            DateTimeFormat.forPattern("yyyy-MM-dd'T'HH:mm:ss.SSSZZ"),
            DateTimeFormat.forPattern("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'"),
            DateTimeFormat.forPattern("EEE MMM dd HH:mm:ss z yyyy") };

    protected static boolean getBoolean(Node child) {
        return Boolean.parseBoolean(child.getFirstChild().getNodeValue());
    }

	protected static Date getDate(Node child, SimpleDateFormat formatter){
        try {
        	return formatter.parse(child.getFirstChild().getNodeValue());
		} catch (Exception e) {
			throw new RuntimeException("Unexpected exception parsing date. Unable to parse simple date: "
                + child.getFirstChild().getNodeValue());
		}
	}
    protected static Date getDate(Node child) {
    	Node firstChild = child.getFirstChild();
        if (firstChild == null) {
            return null;
        }
        
        return parseDate(firstChild.getNodeValue());
    }
    
    protected static Date parseDate(String string) {
        if (string == null) {
            return null;
        }
        for (DateTimeFormatter format : dateFormats) {
            try {
                return format.parseDateTime(string).toDate();
            } catch (Exception e) {
                // do nothing. Tries next DateFormat.
            }
        }
        throw new RuntimeException("Unexpected exception parsing date. Unable to parse date: "
                + string);
    }

    protected static Document getDocumentFromString(String xmlDocument) {
    	return XmlHelper.readXmlDocumentFromString(xmlDocument);
    }

    protected static double getDouble(Node child) {
        return Double.parseDouble(child.getFirstChild().getNodeValue());
    }

    protected static int getInteger(Node child) {
        return Integer.parseInt(child.getFirstChild().getNodeValue());
    }

    protected static long getLong(Node child) {
        return Long.parseLong(child.getFirstChild().getNodeValue());
    }

    protected static String getString(Node child) {
        return child.getFirstChild() != null ? child.getFirstChild().getNodeValue() : "";
    }
}
