package com.indicee.api.services.impl;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;

import com.indicee.api.connector.IndiceeAPIConnector;
import com.indicee.api.connector.MimeType;
import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.connector.util.InputStreamWrapper;
import com.indicee.api.objects.Page;
import com.indicee.api.objects.account.Account;
import com.indicee.api.objects.account.Attribute;
import com.indicee.api.parser.AccountParser;
import com.indicee.api.services.interfaces.AccountService;
import com.indicee.api.util.XmlHelper;

/**
 * This is the implementation of the {@link AccountService}
 * 
 * @author victork
 * 
 */
public class AccountServiceImpl extends IndiceeAPIService implements AccountService {

    private static final String ACCOUNTS_URL = API_URL + "/accounts";
    private static final String ATTRIBUTES_URL = "/attributes";
    private static final String AVATAR_URL = "/avatar";
    private static final String CHILDREN_URL = "/children";
    private static final String PASSWORD_URL = "/password";
    private static final String VALIDATE_URL = "/validate";
    private static final String RESET_URL = "/reset";

    protected AccountServiceImpl() {
        super();
    }

    /**
     * Creates a new account service.
     * 
     * @param connector
     *            The {@link IndiceeAPIConnector} to use.
     */
    public AccountServiceImpl(IndiceeAPIConnector connector) {
        super(connector);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Long createAccount(Account account) throws IndiceeAPIConnectorException {
        String accountID = connector.makePostCallRtnString(ACCOUNTS_URL + CREATE_URL + DOT_XML, account.toString());
        return Long.parseLong(accountID.substring(accountID.indexOf("<account><id>") + 13, accountID
                .indexOf("</id></account>")));
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void deleteAccount(long accountID) throws IndiceeAPIConnectorException {
        connector.makePostCallRtnString(ACCOUNTS_URL + URL_SLASH + accountID + DELETE_URL + DOT_XML);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void deleteAttribute(long accountID, String name) throws IndiceeAPIConnectorException {
        Map<String, String> parameters = new HashMap<String, String>();
        parameters.put("name", name);
        connector.makePostCallRtnString(ACCOUNTS_URL + URL_SLASH + accountID + ATTRIBUTES_URL + DELETE_URL + DOT_XML,
                parameters);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Account getAccount(long accountID) throws IndiceeAPIConnectorException {
    	Document accountDocument = connector.makeGetCallRtnDocument(ACCOUNTS_URL + URL_SLASH + accountID + SHOW_URL + DOT_XML );
        return AccountParser.createAccount(accountDocument);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<Account> getAccountChildren(long accountID) throws IndiceeAPIConnectorException {
    	return getAccountChildren(accountID, new Page(1000, 0));
    	//TODO: fix account children API and then use the code below to get all children. 
//    	Page page = new Page();
//    	List<Account> allList = new ArrayList<Account>();
//        do {
//        	allList.addAll(getAccountChildren(accountID, page));
//        }while(!page.isDone());
//        
//        return allList;
    }
    
    @Override
    public List<Account> getAccountChildren(long accountID, Page page) throws IndiceeAPIConnectorException {
    	Map<String,Object> map = new HashMap<String, Object>();
    	if(page != null) {
    		if(!page.isDefault()) {
    			map.put(PARAM_PAGE, page.getRpp());
    			map.put(PARAM_CURSOR, page.getCursor());
    		}
    	}else{
    		throw new IllegalArgumentException("You have to pass a valid Page to list(String,Page) method.");
    	}
    	Document accountDocument = connector.makeGetCallRtnDocument(ACCOUNTS_URL + URL_SLASH + accountID + CHILDREN_URL
                + SHOW_URL + DOT_XML, map);
        page.setPage(XmlHelper.childText(accountDocument.getDocumentElement(), "next_page"));
        
        List<Account> users = AccountParser.createAccounts(accountDocument);
        return users;
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public List<Account> getAccounts(long accountIDs[]) throws IndiceeAPIConnectorException {
        String userIDsParam = "";
        List<Account> otherAccounts = null;
        for (int i = 0; i < accountIDs.length; i++) {
            if (i == 100) {
                otherAccounts = new ArrayList<Account>(getAccounts(Arrays.copyOfRange(accountIDs, 100,
                        accountIDs.length)));
                break;
            }
            userIDsParam += accountIDs[i] + ",";
        }
        if (!userIDsParam.isEmpty()) {
            userIDsParam = "?account_ids=" + userIDsParam.substring(0, userIDsParam.length() - 1);
        }
        Document accountDocument = connector.makeGetCallRtnDocument(ACCOUNTS_URL + SHOW_URL + DOT_XML + userIDsParam);
        List<Account> users = AccountParser.createAccounts(accountDocument);
        if (otherAccounts != null) {
            users.addAll(otherAccounts);
        }
        return users;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Account getCurrentAccount() throws IndiceeAPIConnectorException {
        List<Account> accounts = getAccounts(new long[] {});
        if (accounts.isEmpty()) {
            throw new IndiceeAPIConnectorException("Error getting user.");
        }
        return accounts.get(0);
    }

    @Override
    protected IndiceeAPIService getInstance(IndiceeAPIConnector connector) {
        return new AccountServiceImpl(connector);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void setAttribute(long accountID, String name, String value, boolean hierarchy)
            throws IndiceeAPIConnectorException {
        connector.makePostCallRtnDocument(ACCOUNTS_URL + URL_SLASH + accountID + ATTRIBUTES_URL + UPDATE_URL + DOT_XML,
                new Attribute(name, value, hierarchy).toString());
    }
    
    @Override
    public List<Attribute> getAttributes(long accountID, boolean includeInherited, String... attributes) throws IndiceeAPIConnectorException {
    	Map<String,Object> parameters = new HashMap<String,Object>();
    	if (attributes.length > 0) {
    		parameters.put("attributes", Arrays.asList(attributes));
    	}
    	if (includeInherited) {
    		parameters.put("inherited", "true");
    	}
    	
    	Document document = connector.makeGetCallRtnDocument(ACCOUNTS_URL + URL_SLASH + accountID + ATTRIBUTES_URL + SHOW_URL + DOT_XML, parameters);
    	return AccountParser.getAttributes(document.getDocumentElement());
    }

    @Override
    public Attribute getAttribute(long accountID, boolean includeInherited, String name) throws IndiceeAPIConnectorException {
    	List<Attribute> attributes = getAttributes(accountID, includeInherited, name);
    	return attributes.size() > 0  ? attributes.get(0) : null;
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public void setAvatar(File avatarImage, MimeType mimeType) throws IndiceeAPIConnectorException {
        Map<String, Object> parameters = new HashMap<String, Object>();
        try {
            parameters.put("file", new InputStreamWrapper(new FileInputStream(avatarImage), mimeType, avatarImage
                    .getName()));
        } catch (FileNotFoundException e) {
            throw new IndiceeAPIConnectorException("Error sending file: " + avatarImage.getName()
                    + " could not find file on local machine.", e);
        }
        connector.makePostCallRtnString(ACCOUNTS_URL + AVATAR_URL + UPLOAD_URL + DOT_XML, parameters);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Account updateAccount(Account account) throws IndiceeAPIConnectorException {
        Document accountDocument = connector.makePostCallRtnDocument(ACCOUNTS_URL + URL_SLASH + account.getId()
                + UPDATE_URL + DOT_XML, account.toString());
        return AccountParser.createAccount(accountDocument);
    }

	@Override public void validatePassword(Account account, String password)
			throws IndiceeAPIConnectorException {
		Map<String,Object> parameters = new HashMap<String, Object>();
		parameters.put("password", password);
		connector.makePostCallRtnString(ACCOUNTS_URL + URL_SLASH + account.getId() + PASSWORD_URL + VALIDATE_URL, parameters);
	}

	@Override
	public void resetPassword(Account account) throws IndiceeAPIConnectorException {
		resetPassword(account, null);
	}
	
	@Override
	public void resetPassword(Account account, String redirectURL)
			throws IndiceeAPIConnectorException {
		Map<String,Object> parameters = new HashMap<String, Object>();
		if(redirectURL!=null){
			parameters.put("redirectURL", redirectURL);
		}
		connector.makePostCallRtnString(ACCOUNTS_URL + URL_SLASH + account.getId() + PASSWORD_URL + RESET_URL, parameters);
		
	}
}
