package com.indicee.api.services.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.indicee.api.connector.IndiceeAPIConnector;
import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.connector.exception.IndiceeAPIStatusException;
import com.indicee.api.exceptions.QualifiedNameSyntaxException;
import com.indicee.api.objects.QualifiedName;
import com.indicee.api.objects.model.Attribute;
import com.indicee.api.objects.model.DataType;
import com.indicee.api.objects.model.Dataset;
import com.indicee.api.objects.model.Entity;
import com.indicee.api.objects.model.Field;
import com.indicee.api.objects.model.Relationship;
import com.indicee.api.objects.model.Field.AggregateType;
import com.indicee.api.objects.model.Field.TrackingType;
import com.indicee.api.objects.model.Hierarchy;
import com.indicee.api.objects.model.Measure;
import com.indicee.api.objects.model.Model;
import com.indicee.api.objects.model.SourceField;
import com.indicee.api.objects.model.Table;
import com.indicee.api.objects.model.binding.Projection;
import com.indicee.api.objects.model.binding.SourceTableBinding;
import com.indicee.api.parser.ModelParser;
import com.indicee.api.parser.ProjectionParser;
import com.indicee.api.parser.RelationshipParser;
import com.indicee.api.services.interfaces.ModelService;
import com.indicee.api.util.XmlHelper;

public class ModelServiceImpl extends IndiceeAPIService implements ModelService {

    private static final String MODEL_URL = "/api/models";
    private static final String ATTRIBUTES_URL = "/attributes";
    private static final String BINDINGS_URL = MODEL_URL + "/bindings";
    private static final String CREATE_URL = "create.xml";
    private static final String EXTEND_URL = "extend.xml";
    private static final String DATASETS_URL = MODEL_URL + "/datasets";
    private static final String DATASETS_V2_URL = MODEL_URL + "/datasets/v2";
    private static final String RELATIONSHIPS_URL = MODEL_URL + "/relationships";
    private static final String HIERARCHIES_URL = MODEL_URL + "/hierarchies";
    private static final String INFOSPACE_URL = "/infospace";
    private static final String DELETE_URL = "delete.xml";
    private static final String ENTITIES_URL = MODEL_URL + "/entities";
    private static final String FIELDS_URL = MODEL_URL + "/fields";
    private static final String KEYS_URL = "/keys";
    private static final String LIST_URL = "list.xml";
    private static final String MAPPINGS_URL = "/mappings";
    private static final String MEASURES_URL = MODEL_URL + "/measures";
    private static final String PROJECTIONS_URL = "/projections";
    private static final String PROPERTIES_URL = "/properties";
    private static final String SHOW_URL = "show.xml";
    private static final String TABLES_URL = "/tables";
    private static final String UPDATE_URL = "update.xml";
    private static final String SOURCE_FIELDS = "/fields";
    private static final String TURBO_URL = "/turbo";
    private static final String SET_URL = "/set";
    private static final String GET_URL = "/get";
	
    protected ModelServiceImpl() {
        super();
    }

    /**
     * Creates a new model service.
     * 
     * @param connector
     *            The {@link IndiceeAPIConnector} to use.
     */
    public ModelServiceImpl(IndiceeAPIConnector connector) {
        super(connector);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String createBinding(SourceTableBinding binding) throws IndiceeAPIConnectorException {
        String rtn = connector.makePostCallRtnString(BINDINGS_URL + URL_SLASH + CREATE_URL, binding.toString());
        return rtn;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public SourceTableBinding getSourceTableBinding(String datasetName, String tableName) throws IndiceeAPIConnectorException {
    	try {
    		Document document = connector.makeGetCallRtnDocument(BINDINGS_URL + URL_SLASH + urlEncode(datasetName) + URL_SLASH + urlEncode(tableName)+ URL_SLASH + SHOW_URL);
    		Element element = document.getDocumentElement();
    		return ModelParser.getBinding(element);
    	} catch (IndiceeAPIStatusException e) {
    		if (e.getStatus() == 404) {
    			return null;
    		}
    		throw e;
    	}
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public String createBindingMapping(String datasetName, String tableName, String fieldName, String projectionName)
            throws IndiceeAPIConnectorException {
        String rtn = connector.makePostCallRtnString(BINDINGS_URL + URL_SLASH + urlEncode(datasetName) + URL_SLASH
                + urlEncode(tableName) + MAPPINGS_URL + URL_SLASH + urlEncode(fieldName) + URL_SLASH
                + urlEncode(projectionName) + URL_SLASH + CREATE_URL);
        return rtn;
    }

    @Override
    public String deleteBindingMapping(String datasetName, String tableName,
    		String fieldName) throws IndiceeAPIConnectorException {
        String rtn = connector.makePostCallRtnString(BINDINGS_URL + URL_SLASH + urlEncode(datasetName) + URL_SLASH
                + urlEncode(tableName) + MAPPINGS_URL + URL_SLASH + urlEncode(fieldName) + URL_SLASH + DELETE_URL);
        return rtn;
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public String createBindingProjection(String datasetName, String tableName, Projection projection)
            throws IndiceeAPIConnectorException {
        String rtn = connector.makePostCallRtnString(BINDINGS_URL + URL_SLASH + urlEncode(datasetName) + URL_SLASH
                + urlEncode(tableName) + PROJECTIONS_URL + URL_SLASH + CREATE_URL, projection.toString());
        return rtn;
    }
    
    @Override
    public String updateBindingProjection(String datasetName, String tableName,
    		Projection projection) throws IndiceeAPIConnectorException {
    	Map<String,Object> params = new HashMap<String, Object>(2);
    	params.put("formula", projection.getFormula());
    	params.put("data_type", projection.getDataType());
        String rtn = connector.makePostCallRtnString(BINDINGS_URL + URL_SLASH + urlEncode(datasetName) + URL_SLASH
                + urlEncode(tableName) + PROJECTIONS_URL + URL_SLASH + urlEncode(projection.getName()) + URL_SLASH + UPDATE_URL, params);
        return rtn;
    }
    
    @Override
    public String deleteBindingProjection(String datasetName, String tableName,
    		String projection) throws IndiceeAPIConnectorException {
        String rtn = connector.makePostCallRtnString(BINDINGS_URL + URL_SLASH + urlEncode(datasetName) + URL_SLASH
                + urlEncode(tableName) + PROJECTIONS_URL + URL_SLASH + urlEncode(projection) + URL_SLASH + DELETE_URL);
        return rtn;
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public String createDataset(Dataset dataset) throws IndiceeAPIConnectorException {
    	//return createDataset(dataset, null);    	
        Document doc = connector.makePostCallRtnDocument(DATASETS_URL + URL_SLASH + CREATE_URL, dataset.toMinimalString());
        return XmlHelper.childText(XmlHelper.getElements(doc.getElementsByTagName("dataset"), "dataset").get(0),"uuid"); // get uuid from xml
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public String createDataset(long infospaceId, Dataset dataset) throws IndiceeAPIConnectorException {
    	Document doc = connector.makePostCallRtnDocument(DATASETS_URL + URL_SLASH + infospaceId + URL_SLASH + CREATE_URL, dataset.toMinimalString());
    	return XmlHelper.childText(XmlHelper.getElements(doc.getElementsByTagName("dataset"), "dataset").get(0),"uuid"); // get uuid from xml
    }
    
    /**
     * {@inheritDoc}
     */
	@Override
	public String createDatasetUsingTemplate(Dataset templateDataset, Dataset newDataset) throws IndiceeAPIConnectorException {

		QualifiedName templateQname;
		try{
			templateQname = new QualifiedName(templateDataset.getQName());
		}catch(QualifiedNameSyntaxException e){
			return null;
		}
		Document doc = connector.makePostCallRtnDocument(DATASETS_URL + URL_SLASH + urlEncode(templateDataset.getName()) + URL_SLASH + EXTEND_URL, newDataset.toString());
	    //after the fb_restInfospace or fb_infospace has been merged into the system, the below line should be used.
		//Document doc = connector.makePostCallRtnDocument(DATASETS_URL + URL_SLASH + templateQname.getSpaceId() + URL_SLASH + urlEncode(templateDataset.getName()) + URL_SLASH + EXTEND_URL, newDataset.toString());
        
        return XmlHelper.childText(XmlHelper.getElements(doc.getElementsByTagName("dataset"), "dataset").get(0),"uuid"); // get uuid from xml
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean createDatasetAttribute(String datasetName, Attribute attribute) throws IndiceeAPIConnectorException {
        connector.makePostCallRtnString(DATASETS_URL + URL_SLASH + urlEncode(datasetName)
                + ATTRIBUTES_URL + URL_SLASH + CREATE_URL, attribute.toString());
        return true;
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public boolean createDatasetAttribute(long infospaceId, String datasetName, Attribute attribute) throws IndiceeAPIConnectorException {
        connector.makePostCallRtnString(DATASETS_URL + URL_SLASH + infospaceId + URL_SLASH + urlEncode(datasetName)
                + ATTRIBUTES_URL + URL_SLASH + CREATE_URL, attribute.toString());
        return true;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String createDatasetTable(String datasetName, Table table) throws IndiceeAPIConnectorException {
        Document doc = connector.makePostCallRtnDocument(DATASETS_URL + URL_SLASH + urlEncode(datasetName)
                + TABLES_URL + URL_SLASH + CREATE_URL, table.toString());
        return XmlHelper.childText(XmlHelper.getElements(doc.getElementsByTagName("table"), "table").get(0),"uuid");
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public String createDatasetTable(long infospaceId, String datasetName, Table table) throws IndiceeAPIConnectorException {
       	Document doc = connector.makePostCallRtnDocument(DATASETS_URL + URL_SLASH + infospaceId + URL_SLASH + urlEncode(datasetName)
                + TABLES_URL + URL_SLASH + CREATE_URL, table.toString());
        return XmlHelper.childText(XmlHelper.getElements(doc.getElementsByTagName("table"), "table").get(0),"uuid");

    }

    @Override
    public String createDatasetTableField(String datasetName, String tableName, SourceField sourceField)
    		throws IndiceeAPIConnectorException {
    	Document doc = connector.makePostCallRtnDocument(DATASETS_URL + URL_SLASH + urlEncode(datasetName)
                + TABLES_URL + URL_SLASH + urlEncode(tableName) + SOURCE_FIELDS +  URL_SLASH
                + CREATE_URL, sourceField.toString());
        return XmlHelper.childText(XmlHelper.getElements(doc.getElementsByTagName("source_field"), "source_field").get(0),"uuid");
    }
    
    @Override
    public String createDatasetTableField(long infospaceId, String datasetName, String tableName, SourceField sourceField)
    		throws IndiceeAPIConnectorException {
    	Document doc = connector.makePostCallRtnDocument(DATASETS_URL + URL_SLASH + infospaceId + URL_SLASH + urlEncode(datasetName)
                + TABLES_URL + URL_SLASH + urlEncode(tableName) + SOURCE_FIELDS +  URL_SLASH
                + CREATE_URL, sourceField.toString());
        return XmlHelper.childText(XmlHelper.getElements(doc.getElementsByTagName("source_field"), "source_field").get(0),"uuid");
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public String createEntity(Entity entity) throws IndiceeAPIConnectorException {
        Document doc = connector.makePostCallRtnDocument(ENTITIES_URL + URL_SLASH + CREATE_URL, entity.toString());
        return XmlHelper.childText(XmlHelper.getElements(doc.getElementsByTagName("entity"), "entity").get(0),"uuid"); 
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String createEntityAttribute(String entityName, Attribute attribute) throws IndiceeAPIConnectorException {
        String rtn = connector.makePostCallRtnString(ENTITIES_URL + URL_SLASH + urlEncode(entityName) + ATTRIBUTES_URL
                + URL_SLASH + CREATE_URL, attribute.toString());
        return rtn;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String createEntityKey(String entityName, String fieldName) throws IndiceeAPIConnectorException {
        String rtn = connector.makePostCallRtnString(ENTITIES_URL + URL_SLASH + urlEncode(entityName) + KEYS_URL
                + URL_SLASH + urlEncode(fieldName) + URL_SLASH + CREATE_URL);
        return rtn;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String createEntityProperty(String entityName, String fieldName) throws IndiceeAPIConnectorException {
        String rtn = connector.makePostCallRtnString(ENTITIES_URL + URL_SLASH + urlEncode(entityName) + PROPERTIES_URL
                + URL_SLASH + urlEncode(fieldName) + URL_SLASH + CREATE_URL);
        return rtn;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String createField(Field field) throws IndiceeAPIConnectorException {
        Document doc = connector.makePostCallRtnDocument(FIELDS_URL + URL_SLASH + CREATE_URL, field.toString());
        return XmlHelper.childText(XmlHelper.getElements(doc.getElementsByTagName("field"), "field").get(0),"uuid"); // get uuid from xml
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public String createHierarchy(Hierarchy hierarchy) throws IndiceeAPIConnectorException {
        Document doc = connector.makePostCallRtnDocument(HIERARCHIES_URL + URL_SLASH + CREATE_URL, hierarchy.toString());
        return XmlHelper.childText(XmlHelper.getElements(doc.getElementsByTagName("hierarchy"), "hierarchy").get(0),"uuid"); // get uuid from xml
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String createMeasure(Measure measure) throws IndiceeAPIConnectorException {
        Document doc = connector.makePostCallRtnDocument(MEASURES_URL + URL_SLASH + CREATE_URL, measure.toString());
        return XmlHelper.childText(XmlHelper.getElements(doc.getElementsByTagName("measure"), "measure").get(0),"uuid"); 
    }    
    
    @Override
    public void createRelationship(Relationship relationship) throws IndiceeAPIConnectorException {
    	String body = relationship.toString();
		connector.makePostCallRtnString(RELATIONSHIPS_URL + URL_SLASH + CREATE_URL, body);
    }

	@Override
	public void deleteBinding(String datasetName, String tableName) throws IndiceeAPIConnectorException{
		connector.makePostCallRtnString(BINDINGS_URL + URL_SLASH + urlEncode(datasetName) + URL_SLASH + tableName + URL_SLASH + DELETE_URL);
	}
	
    /**
     * {@inheritDoc}
     */
    @Override
    public void deleteDataset(String datasetName) throws IndiceeAPIConnectorException {
    	connector.makePostCallRtnString(DATASETS_URL + URL_SLASH + urlEncode(datasetName) + URL_SLASH  + DELETE_URL);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void deleteDatasetAttribute(String datasetName, String attributeName) throws IndiceeAPIConnectorException {
        connector.makePostCallRtnString(DATASETS_URL + URL_SLASH + urlEncode(datasetName)
                + ATTRIBUTES_URL + URL_SLASH + urlEncode(attributeName) + URL_SLASH + DELETE_URL);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void deleteDatasetTable(String datasetName, String tableName) throws IndiceeAPIConnectorException {
        connector.makePostCallRtnString(DATASETS_URL + URL_SLASH + urlEncode(datasetName)
                + TABLES_URL + URL_SLASH + urlEncode(tableName) + URL_SLASH + DELETE_URL);
    }

	@Override
	public void deleteEntity(String entityNameOrUUID) throws IndiceeAPIConnectorException{
		connector.makePostCallRtnString(ENTITIES_URL + URL_SLASH + urlEncode(entityNameOrUUID) + URL_SLASH + DELETE_URL);
	}
	
	/**
     * {@inheritDoc}
     */
    @Override
    public void deleteEntityAttribute(String entityName, String attributeName) throws IndiceeAPIConnectorException {
        connector.makePostCallRtnString(ENTITIES_URL + URL_SLASH + urlEncode(entityName) + ATTRIBUTES_URL
                + URL_SLASH + urlEncode(attributeName) + URL_SLASH + DELETE_URL);
    }
    
    @Override
	public void deleteField(String fieldNameOrUUID) throws IndiceeAPIConnectorException{
		connector.makePostCallRtnString(FIELDS_URL + URL_SLASH + urlEncode(fieldNameOrUUID) + URL_SLASH + DELETE_URL);
	}
	
	@Override 
	public void deleteHierarchy(String hierarchyNameOrUUID) throws IndiceeAPIConnectorException{
		connector.makePostCallRtnString(HIERARCHIES_URL + URL_SLASH + urlEncode(hierarchyNameOrUUID) + URL_SLASH + DELETE_URL);
	}
	
	@Override
	public void deleteMeasure(String measureNameOrUUID) throws IndiceeAPIConnectorException{
		connector.makePostCallRtnString(MEASURES_URL + URL_SLASH + urlEncode(measureNameOrUUID) + URL_SLASH +  DELETE_URL);
	}

	@Override
    public void deleteRelationship(Relationship relationship)
    		throws IndiceeAPIConnectorException {
    	String body = relationship.toString();
		connector.makePostCallRtnString(RELATIONSHIPS_URL + URL_SLASH + DELETE_URL, body);
    }


    /**
     * {@inheritDoc}
     */
    @Override
    public Dataset getDataset(String uuidOrName) throws IndiceeAPIConnectorException {
        Document datasetDocument;
        datasetDocument = connector.makeGetCallRtnDocument(DATASETS_URL + URL_SLASH + urlEncode(uuidOrName) + URL_SLASH
                + SHOW_URL);
        return ModelParser.createDataset(datasetDocument);
    }
    
    /**
     * {@inheritDoc}
     */
	@Override
	public Dataset getDataset(String datasetUUIDOrName, long infospaceId)
			throws IndiceeAPIConnectorException {
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("infospaceId", infospaceId );
		Document datasetDocument = connector.makeGetCallRtnDocument(
				DATASETS_URL + URL_SLASH + urlEncode(datasetUUIDOrName) + URL_SLASH + SHOW_URL, 
				map);
		return ModelParser.createDataset(datasetDocument);
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public Table getDatasetTable(String datasetUUIDOrName, String tableName) throws IndiceeAPIConnectorException {
        Document datasetTableDocument = connector.makeGetCallRtnDocument(DATASETS_URL + URL_SLASH
                + urlEncode(datasetUUIDOrName) + TABLES_URL + URL_SLASH + urlEncode(tableName) + URL_SLASH + SHOW_URL);
        return ModelParser.createDatasetTable(datasetTableDocument);
    }
    
    @Override
    public SourceField getDatasetTableField(String datasetUUIDOrName, String tableUUIDOrName, String sourceFieldUUIDOrNaString) 
        	throws IndiceeAPIConnectorException{
    	Document datasetTableDocument = connector.makeGetCallRtnDocument(DATASETS_URL + URL_SLASH
                + urlEncode(datasetUUIDOrName) + TABLES_URL + URL_SLASH + urlEncode(tableUUIDOrName)
                + SOURCE_FIELDS + URL_SLASH +  urlEncode(sourceFieldUUIDOrNaString) + URL_SLASH + SHOW_URL);
        return ModelParser.createDatasetTableField(datasetTableDocument);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<Entity> listEntities() throws IndiceeAPIConnectorException {
        Document entityDocument;
        entityDocument = connector.makeGetCallRtnDocument(ENTITIES_URL + URL_SLASH + LIST_URL);
        return ModelParser.entityParser.parseList(entityDocument);
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public Entity getEntity(String uuidOrName) throws IndiceeAPIConnectorException {
        Document entityDocument;
        entityDocument = connector.makeGetCallRtnDocument(ENTITIES_URL + URL_SLASH + urlEncode(uuidOrName) + URL_SLASH
                + SHOW_URL);
        return ModelParser.entityParser.parse(entityDocument);
    }

    
    @Override
    public List<Field> listFields() throws IndiceeAPIConnectorException {
        Document fieldsDocument;
        fieldsDocument = connector.makeGetCallRtnDocument(FIELDS_URL + URL_SLASH + LIST_URL);
        return ModelParser.fieldParser.parseList(fieldsDocument);
    }
    
    @Override
    public Field getField(String uuidOrName) throws IndiceeAPIConnectorException {
    	try {
        	Document fieldDocument;
        	fieldDocument = connector.makeGetCallRtnDocument(FIELDS_URL + URL_SLASH + urlEncode(uuidOrName) + URL_SLASH
                + SHOW_URL);
        	return ModelParser.fieldParser.parse(fieldDocument);
    	} catch (IndiceeAPIStatusException e) {
    		if (e.getStatus() == 404) {
    			return null;
    		}
    		throw e;
    	}
    }
    
    @Override
    public Hierarchy getHierarchy(String uuidOrName) throws IndiceeAPIConnectorException {
        Document hierarchyDocument;
        hierarchyDocument = connector.makeGetCallRtnDocument(FIELDS_URL + URL_SLASH + urlEncode(uuidOrName) + URL_SLASH
                + SHOW_URL);
        return ModelParser.createHierarchy(hierarchyDocument);
    }

    @Override
    public List<Measure> listMeasures() throws IndiceeAPIConnectorException {
        Document measuresDocument = connector.makeGetCallRtnDocument(MEASURES_URL + URL_SLASH + LIST_URL);
        return ModelParser.measureParser.parseList(measuresDocument);
    }
    
    @Override
    public Measure getMeasure(String uuidOrName) throws IndiceeAPIConnectorException {
        Document measureDocument;
        measureDocument = connector.makeGetCallRtnDocument(MEASURES_URL + URL_SLASH + urlEncode(uuidOrName) + URL_SLASH + SHOW_URL);
        return ModelParser.measureParser.parse(measureDocument);
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public Model getModel() throws IndiceeAPIConnectorException {
        return getModel(null);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Model getModel(Long infospaceId) throws IndiceeAPIConnectorException {
    	Map<String,Object> map = new HashMap<String, Object>();
    	if(infospaceId!=null){
    		map.put("spaceId", infospaceId);
    	}
        Document datasetDocument;
        datasetDocument = connector.makeGetCallRtnDocument(MODEL_URL + URL_SLASH + SHOW_URL, map);
        return ModelParser.createModel(datasetDocument);
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public List<Dataset> listDatasets() throws IndiceeAPIConnectorException {
        Document datasetListDocument = connector.makeGetCallRtnDocument(DATASETS_URL + URL_SLASH + LIST_URL);
        return ModelParser.createDatasetList(datasetListDocument);
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public List<Dataset> listDatasetsWithInfospace(long infospaceId) throws IndiceeAPIConnectorException{
        Document datasetListDocument = connector.makeGetCallRtnDocument(DATASETS_URL + URL_SLASH + infospaceId + URL_SLASH + LIST_URL);
        return ModelParser.createDatasetList(datasetListDocument);
    }
    

    /**
     * {@inheritDoc}
     */
    @Override
    public List<Table> listDatasetTables(String datasetUUIDOrName) throws IndiceeAPIConnectorException {
        Document datasetTablesListDocument = connector.makeGetCallRtnDocument(DATASETS_URL + URL_SLASH
                + urlEncode(datasetUUIDOrName) + TABLES_URL + URL_SLASH + LIST_URL);
        return ModelParser.createDatasetTablesList(datasetTablesListDocument);
    }
    
    @Override
    public List<Hierarchy> listHierarchies() throws IndiceeAPIConnectorException{
    	Document doc = connector.makeGetCallRtnDocument(HIERARCHIES_URL + URL_SLASH + LIST_URL);
    	return ModelParser.createHierarchyList(doc);
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public List<Projection> listProjections(String datasetName, String tableName)
    		throws IndiceeAPIConnectorException {
    	try {
        	Document document = connector.makeGetCallRtnDocument(BINDINGS_URL + URL_SLASH + urlEncode(datasetName) + URL_SLASH
                + urlEncode(tableName) + PROJECTIONS_URL + URL_SLASH + LIST_URL);
        	Element element = document.getDocumentElement();
    		return ProjectionParser.parseList(element);
    	} catch (IndiceeAPIStatusException e) {
    		if (e.getStatus() == 404) {
    			return Collections.emptyList();
    		}
    		throw e;
    	}
    }

    /**
     * {@inheritDoc}
     * @throws IndiceeAPIConnectorException 
     */
    @Override
    public List<Relationship> listRelationships() throws IndiceeAPIConnectorException {
    	Document document = connector.makeGetCallRtnDocument(RELATIONSHIPS_URL + URL_SLASH + LIST_URL);
    	Element element = document.getDocumentElement();
		return RelationshipParser.parseRelationships(element );
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public String setModel(String modelString) throws IndiceeAPIConnectorException {
        return connector.makePostCallRtnString(MODEL_URL + URL_SLASH + CREATE_URL, modelString);
    }    

    
    /**
     * {@inheritDoc}
     */
    @Override
    public String updateDataset(String uuidOrName, String newDescription) throws IndiceeAPIConnectorException {
        HashMap<String, Object> map = new HashMap<String, Object>();
        map.put("description", newDescription);
        String rtn = connector.makePostCallRtnString(DATASETS_URL + URL_SLASH + urlEncode(uuidOrName) + URL_SLASH
                + UPDATE_URL, map);
        return rtn;
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public String updateField(Field field) throws IndiceeAPIConnectorException {
    	String name = field.getName();
    	Map<String, Object> map = new HashMap<String, Object>();
    	DataType dataType = field.getDataType();
    	if (dataType != null) {
    		map.put("data_type", dataType);
    	}
    	TrackingType trackingType = field.getTrackingType();
    	if (trackingType != null) {
    		map.put("tracking_type", trackingType);
    	}
    	AggregateType aggregationType = field.getAggregateType();
    	if (aggregationType != null) {
    		map.put("aggregate_type", aggregationType);
    	}
    	String formula = field.getFormula();
    	if (formula != null) {
    		map.put("formula", formula);
    	}

        Document doc = connector.makePostCallRtnDocument(FIELDS_URL + URL_SLASH + urlEncode(name) + URL_SLASH +  UPDATE_URL, map);
        return XmlHelper.childText(XmlHelper.getElements(doc.getElementsByTagName("field"), "field").get(0),"uuid"); // get uuid from xml
    }
    
    @Override
    public void createFieldAttribute(String fieldName, Attribute attribute) throws IndiceeAPIConnectorException {
    	String xmlString = attribute.toString();
    	connector.makePostCallRtnString(FIELDS_URL + URL_SLASH + urlEncode(fieldName) + ATTRIBUTES_URL + URL_SLASH + CREATE_URL, xmlString);	
    }
    
    @Override
    protected IndiceeAPIService getInstance(IndiceeAPIConnector connector) {
        return new ModelServiceImpl(connector);
    }
    
	@Override
	public String setTurboMode(String qname, boolean active) throws IndiceeAPIConnectorException {
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("qname", qname);
		map.put("active", active);
		return connector.makePostCallRtnString(DATASETS_V2_URL + TURBO_URL + SET_URL + DOT_XML , map);
	}

	@Override
	public boolean getTurboMode(String qname) throws IndiceeAPIConnectorException {
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("qname", qname);
		String result = connector.makeGetCallRtnString(DATASETS_V2_URL + TURBO_URL + GET_URL + DOT_XML, map);
		return Boolean.parseBoolean(result);
	}
	
	/**
	 * @see com.indicee.api.services.interfaces.ModelService#createAncillaryField(java.lang.String, com.indicee.api.objects.model.SourceField)
	 */
	@Override
	public String createAncillaryField(String datasetName, SourceField field) throws IndiceeAPIConnectorException {
		String name = field.getName();
		if (name == null || name.length() == 0) {
			throw new IllegalStateException("No name set on field");
		}
		DataType type = field.getDataType();
		if (type == null) {
			throw new IllegalStateException("No DataType set on field");
		}
		List<Attribute> attributes = field.getAttributes();
		if (attributes == null) {
			attributes = new ArrayList<Attribute>(2);
			field.setAttributes(attributes);
		}
		
		// we need a display name, if one isn't set use the name
		Attribute displayName = findAttribute(attributes, "displayName");
		if (displayName == null) {
			attributes.add(new Attribute("displayName", Attribute.Type.String, name));
		}

		// we need a complexType, if one isn't set create from data type
		Attribute complexType = findAttribute(attributes, "complexType");
		if (complexType == null) {
			attributes.add(new Attribute("complexType", Attribute.Type.String, toComplexType(type)));
		}
		
		String xml = field.toString();
		Document document = connector.makePostCallRtnDocument(DATASETS_URL + URL_SLASH + urlEncode(datasetName) + SOURCE_FIELDS + URL_SLASH + CREATE_URL, xml);
		Element element = document.getDocumentElement();
		SourceField copy = ModelParser.getSourceField(element);
		return copy.getUUID();
	}
	
	private static String toComplexType(DataType type) {
		switch (type) {
		case DATE: 
			return "Date";
			
		case NUMBER: 
			return "Number";
			
		case STRING: 
			return "Text";
		default:
			throw new IllegalArgumentException("No conversion for: " + type);
		}
	}

	private static Attribute findAttribute(List<Attribute> attributes, String name) {
		for (Attribute attribute : attributes) {
			if (name.equals(attribute.getName())) {
				return attribute;
			}
		}

		return null;
	}
/*

        Count("Count", 7), Date("Date", 3), Integer("Integer", 6), List("Text from List only", 1), Number("Number", 5), Options(
                "Text with Options", 2), Text("Text", 0), Timezone("ISO Timezone", 4);

 */

}
