package com.indicee.api.services.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.indicee.api.connector.IndiceeAPIConnector;
import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.connector.exception.IndiceeAPIStatusException;
import com.indicee.api.objects.Page;
import com.indicee.api.objects.ViewWindow;
import com.indicee.api.objects.dashboard.CachingMetadata;
import com.indicee.api.objects.report.Metadatas;
import com.indicee.api.objects.report.Report;
import com.indicee.api.objects.report.ReportShareScope;
import com.indicee.api.objects.report.RequestStatus;
import com.indicee.api.objects.report.Results;
import com.indicee.api.objects.report.axes.Axes;
import com.indicee.api.objects.report.data.Data;
import com.indicee.api.objects.report.view.View;
import com.indicee.api.parser.ReportParser;
import com.indicee.api.services.interfaces.ReportServiceV3;
import com.indicee.api.util.XmlHelper;

public class ReportServiceV3Impl extends IndiceeAPIService implements ReportServiceV3 {

    private static final String AXES_URL = "/axes";
    private static final String DATA_URL = "/data";
    private static final String RESULTS_URL = "/results";
    private static final String METADATA_URL = "/metadata";
    private static final String REPORT_URL = API_URL + "/reports/v3";
    private static final String REQUEST_URL = "/request";
    private static final String GROUPS_URL = "/groups/";
    private static final String SHARE_URL = "/share";
    private static final String ALL_URL = "/all";
    private static final String UNSHARE_URL = "/unshare";

    protected ReportServiceV3Impl() {
        super();
    }

    /**
     * Creates a new report service.
     * 
     * @param connector
     *            The {@link IndiceeAPIConnector} to use.
     */
    public ReportServiceV3Impl(IndiceeAPIConnector connector) {
        super(connector);
    }

    @Override
    public Report createReport(String name) throws IndiceeAPIConnectorException {
        return createReport(name, null);
    }

    @Override
    public Report createReport(String name, View view) throws IndiceeAPIConnectorException {
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("name", name);
        if (view != null) {
            params.put("view", view.toFormatedString());
        }
        Document reportDocument = connector.makePostCallRtnDocument(REPORT_URL + CREATE_URL, params);
        return ReportParser.createReport(reportDocument);
    }

    @Override
    public Report deleteReport(String qname) throws IndiceeAPIConnectorException {
        Document reportDocument = connector.makePostCallRtnDocument(REPORT_URL + DELETE_URL + URL_QUERY_PARAM_START
                + "qname=" + urlEncode(qname));
        return ReportParser.createReport(reportDocument);
    }

    @Override
    protected IndiceeAPIService getInstance(IndiceeAPIConnector connector) {
        return new ReportServiceV3Impl(connector);
    }

    @Override
    public Report getReport(String qname) throws IndiceeAPIConnectorException {
        return getReport(qname, new ViewWindow());
    }

    @Override
    public Report getReport(String qname, ViewWindow viewWindow) throws IndiceeAPIConnectorException {
        Map<String, Object> params = new HashMap<String, Object>();
        if (viewWindow != null) {
        	params.put("window", viewWindow);
        }
        params.put("qname", qname);
        Document reportDocument = connector.makeGetCallRtnDocument(REPORT_URL, params);
        return ReportParser.createReport(reportDocument);
    }

    @Override
    public Axes getReportAxes(View reportView) throws IndiceeAPIConnectorException {
        return getReportAxes(reportView, new ViewWindow());
    }

    @Override
    public Axes getReportAxes(View reportView, ViewWindow viewWindow) throws IndiceeAPIConnectorException {
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("window", viewWindow);
        params.put("view", reportView.toString());
        Document reportDocument = connector.makePostCallRtnDocument(REPORT_URL + AXES_URL, params);
        return ReportParser.createReportAxes(reportDocument);
    }

    @Override
    public Data getReportData(View reportView) throws IndiceeAPIConnectorException {
        return getReportData(reportView, new ViewWindow());
    }

    @Override
    public Data getReportData(View reportView, ViewWindow viewWindow) throws IndiceeAPIConnectorException {
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("window", viewWindow);
        params.put("view", reportView.toString());
        Document reportDocument = connector.makePostCallRtnDocument(REPORT_URL + DATA_URL, params);
        return ReportParser.createReportData(reportDocument);
    }

    @Override
    public Metadatas getReportMetadata(View reportView) throws IndiceeAPIConnectorException {
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("view", reportView.toString());
        Document reportDocument = connector.makePostCallRtnDocument(REPORT_URL + METADATA_URL, params);
        return ReportParser.createReportMetadatas(reportDocument);
    }

    @Override
    public List<Report> listReports(Page page) throws IndiceeAPIConnectorException {
    	return listReportsByInfospace(null, page);
    }
    
    @Override
    public List<Report> listReports() throws IndiceeAPIConnectorException {
    	return listReportsByInfospace(null);
    }
    
    @Override
    public List<Report> listReportsByInfospace(Long infospaceId, Page page)
    		throws IndiceeAPIConnectorException {
    	Map<String,Object> map = new HashMap<String, Object>();
    	if(page != null) {
    		if(!page.isDefault()) {
    			map.put(PARAM_PAGE, page.getRpp());
    			map.put(PARAM_CURSOR, page.getCursor());
    		}
    	}else{
    		throw new IllegalArgumentException("You have to pass a valid Page to list(String,Page) method.");
    	}
    	
    	if(infospaceId!=null){
    		map.put("infospaceId", infospaceId);
    	}
    	
    	Document reportsDocument = connector.makeGetCallRtnDocument(REPORT_URL + LIST_URL, map);
        page.setPage(XmlHelper.childText(reportsDocument.getDocumentElement(), "next_page")); 
        return ReportParser.createReports(reportsDocument);
    }
    
    @Override
    public List<Report> listReportsByInfospace(Long infospaceId)
    		throws IndiceeAPIConnectorException {
    	Page page = new Page();
    	List<Report> allList = new ArrayList<Report>();
        do {
        	allList.addAll(listReportsByInfospace(infospaceId, page));
        }while(!page.isDone());        
        return allList;
    }
    
    @Override
    public List<Report> listReportsByGroup(long groupId)
    		throws IndiceeAPIConnectorException {
        Document reportDocument = connector.makeGetCallRtnDocument(REPORT_URL + GROUPS_URL + groupId + LIST_URL);
        return ReportParser.createReports(reportDocument);
    }

    @Override
    public Report updateReport(String qname, String name, View view) throws IndiceeAPIConnectorException {
        return updateReport(qname, name, view, null);
    }
    
    @Override
    public Report updateReport(String qname, String name, View view, CachingMetadata caching) throws IndiceeAPIConnectorException {
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("qname", qname);
        if (name != null) {
        	params.put("name", name);
    	}
        if (view != null) {
            params.put("view", view.toFormatedString());
        }
        if (caching != null) {
            params.put("caching_metadata", caching.toFormatedString());
        }
        Document reportDocument = connector.makePostCallRtnDocument(REPORT_URL + UPDATE_URL, params);
        return ReportParser.createReport(reportDocument);
    }
    
    @Override
    public String requestReport(View view, ViewWindow viewWindow, Long ttl) throws IndiceeAPIConnectorException{
    	Map<String, Object> map = new HashMap<String, Object>();
    	map.put("view", view);
    	if(viewWindow !=null) {
    		map.put("window", viewWindow);
    	}
    	if(ttl != null) {
    		map.put("ttl", ttl);
    	}
    	Document doc = connector.makePostCallRtnDocument(REPORT_URL + REQUEST_URL ,map);
    	return XmlHelper.childText(XmlHelper.getElements(doc.getElementsByTagName("query_request"), "query_request").get(0),"request_id"); 
    }
    
    @Override
    public String requestReport(String qname, ViewWindow viewWindow, Long ttl) throws IndiceeAPIConnectorException {
    	Map<String, Object> map = new HashMap<String, Object>();
    	map.put("qname", qname);
    	if(viewWindow !=null) {
    		map.put("window", viewWindow);
    	}
    	if(ttl != null) {
    		map.put("ttl", ttl);
    	}
    	Document doc = connector.makePostCallRtnDocument(REPORT_URL + REQUEST_URL ,map);
    	return XmlHelper.childText(XmlHelper.getElements(doc.getElementsByTagName("query_request"), "query_request").get(0),"request_id"); 
    }

    @Override
    public String requestReport(View view, Long ttl) throws IndiceeAPIConnectorException {
    	return requestReport(view, null, ttl);
    }
    
    @Override
    public String requestReport(View view, ViewWindow viewWindow) throws IndiceeAPIConnectorException{
    	return requestReport(view, viewWindow, null);
    }
    
    @Override
    public String requestReport(View view) throws IndiceeAPIConnectorException{
    	return requestReport(view, null, null);    	
    }
    
    @Override
    public RequestStatus requestStatus(String requestId) throws IndiceeAPIConnectorException {
    	try {
	    	Document document = connector.makeGetCallRtnDocument(REPORT_URL + REQUEST_URL + URL_SLASH + requestId);
	    	Element element = document.getDocumentElement();
	    	String status = XmlHelper.childText(element, "status");
	    	return RequestStatus.valueOf(status);
    	} catch (IndiceeAPIStatusException e) {
    		if (404 == e.getStatus()) {
    			return null;
    		}
    		throw e;
    	}
    }
    
    @Override
    public void requestCancel(String requestId) throws IndiceeAPIConnectorException {
    	connector.makeDeleteCallRtnString(REPORT_URL + REQUEST_URL + URL_SLASH + requestId);
    }
    
    @Override
    public Data requestData(String requestId) throws IndiceeAPIConnectorException{
    	Document document = connector.makeGetCallRtnDocument(REPORT_URL + REQUEST_URL + URL_SLASH + requestId + DATA_URL);
    	return ReportParser.createReportData(document);
    }  
    
    @Override
    public Results requestResults(String requestId) throws IndiceeAPIConnectorException {
    	Document document = connector.makeGetCallRtnDocument(REPORT_URL + REQUEST_URL + URL_SLASH + requestId + RESULTS_URL);
    	Axes axes = ReportParser.createReportAxes(document);
		Data data = ReportParser.createReportData(document);
		return new Results(axes, data, ReportParser.createReportGeneratedOn(document), 
				ReportParser.createReportTimeToGenerate(document));
    }

	@Override
	public void shareReport(String qName,  long groupId, ReportShareScope scope)
			throws IndiceeAPIConnectorException {
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("qname", qName);
		map.put("scope", scope.getScope());
		map.put("group_id", groupId);
		connector.makePostCallRtnString(REPORT_URL + SHARE_URL ,map);
	}

	@Override
	public void shareAllReports(long groupId, ReportShareScope scope)
			throws IndiceeAPIConnectorException {
		shareAllReports(groupId, scope, false);
	}
	
	@Override
	public void shareAllReports(long groupId, ReportShareScope scope, boolean force)
			throws IndiceeAPIConnectorException {
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("scope", scope.getScope());
		map.put("group_id", groupId);
		map.put("force", force);

		connector.makePostCallRtnString(REPORT_URL + SHARE_URL + ALL_URL, map);
	}
	
	@Override
	public void unshareReport(String qName, long groupId, ReportShareScope scope)
			throws IndiceeAPIConnectorException {
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("qname", qName);
		map.put("scope", scope.getScope());
		map.put("group_id", groupId);
		connector.makePostCallRtnString(REPORT_URL + UNSHARE_URL ,map);
	}
	
	@Override
	public void unshareAllReports(long groupId, ReportShareScope scope)
			throws IndiceeAPIConnectorException {
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("scope", scope.getScope());
		map.put("group_id", groupId);
		connector.makePostCallRtnString(REPORT_URL + UNSHARE_URL + ALL_URL, map);
	}
}
