package com.indicee.api.services.interfaces;

import java.io.File;
import java.util.List;

import com.indicee.api.connector.MimeType;
import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.objects.Page;
import com.indicee.api.objects.account.Account;
import com.indicee.api.objects.account.Attribute;

/**
 * The user service is use to create, update and view users from Indicee.
 * 
 */
public interface AccountService {

    /**
     * Creates a new user.
     * 
     * @param account
     *            The new user to create
     * @return A user object that only contains data for the new user id.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Long createAccount(Account account) throws IndiceeAPIConnectorException;

    void deleteAccount(long accountID) throws IndiceeAPIConnectorException;

    void deleteAttribute(long accountID, String name) throws IndiceeAPIConnectorException;

    /**
     * Gets the user with the given ID.
     * 
     * @param accountID
     *            The id of the user to get.
     * @return The user with the given ID
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Account getAccount(long accountID) throws IndiceeAPIConnectorException;

    List<Account> getAccountChildren(long accountId) throws IndiceeAPIConnectorException;
    List<Account> getAccountChildren(long accountId, Page page) throws IndiceeAPIConnectorException;
    
    /**
     * Gets a list of users.
     * 
     * @param accountIDs
     *            The ids of the users to get
     * @return The list of users
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public List<Account> getAccounts(long[] accountIDs) throws IndiceeAPIConnectorException;

    /**
     * Gets the account belonging to consumer key's user.
     * 
     * @return Account
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Account getCurrentAccount() throws IndiceeAPIConnectorException;

    /**
     * Set an account's attribute value, if it doesn't exist it's created otherwise it's updated.
     * @param accountID - the ID of the account
     * @param name - the name of the attribute
     * @param value - the value for the attribute
     * @param hierarchy - true for the attribute's value to inheritted by child accounts.
     * @throws IndiceeAPIConnectorException
     */
    void setAttribute(long accountID, String name, String value, boolean hierarchy) throws IndiceeAPIConnectorException;
    
    /**
     * Retrieve a set of attributes set on an account. 
     * @param accountID - the ID of the account
     * @param includeInherited - set to true to include attributes inherited from the account's ancestors
     * @param names - optional the names of the attributes wanted (empty means all).
     * @return List of Attribute
     * @throws IndiceeAPIConnectorException
     */
    List<Attribute> getAttributes(long accountID, boolean includeInherited, String... names) throws IndiceeAPIConnectorException;
    
    /**
     * Retrieve an individual attribute of an account
     * @param accountID - the ID of the account
     * @param includeInherited - set to true to include attributes inherited from the account's ancestors
     * @param name - the name of the required attribute
     * @return Attribute
     * @throws IndiceeAPIConnectorException
     */
    Attribute getAttribute(long accountID, boolean includeInherited, String name) throws IndiceeAPIConnectorException;

    /**
     * Upload an image to use for the consumer key's user avatar 
     * @param avatarImage
     * @param mimeType
     * @throws IndiceeAPIConnectorException
     */
    void setAvatar(File avatarImage, MimeType mimeType) throws IndiceeAPIConnectorException;

    /**
     * Updates an existing user, the consumer key needs to be for an administrator of the Account.
     * 
     * @param account - The user to update.
     * @return A user object that only contains data for the updated user id.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Account updateAccount(Account account) throws IndiceeAPIConnectorException;
    
    /**
     * Validate that the password passed is correct for the specified user, 
     * the consumer key needs to be for an administrator of the Account.
     * @param account
     * @param password
     * @throws IndiceeAPIConnectorException
     */
    public void validatePassword(Account account, String password) throws IndiceeAPIConnectorException;
    
    /**
     * Request for a reset password email to be sent to the owner of the account, the consumer key 
     * needs to be for an administrator of the Account. 
     * @param account
     * @param redirectURL - The URL the user is redirected to after changeing their password.
     * @throws IndiceeAPIConnectorException
     */
    public void resetPassword(Account account, String redirectURL) throws IndiceeAPIConnectorException;
    
    /**
     * Request for a reset password email to be sent to the owner of the account, the consumer key 
     * needs to be for an administrator of the Account. 
     * @param account
     * @throws IndiceeAPIConnectorException
     */
    public void resetPassword(Account account) throws IndiceeAPIConnectorException;

}
