package com.indicee.api.services.interfaces;

import java.util.List;

import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.objects.auth.Consumer;
import com.indicee.api.objects.auth.Token;

/**
 * Service for working with applications, so creating consumers and tokens, this is used for creating applications on behalf of users.
 */
public interface AuthService {

    /**
     * Creates an application, so consumer key and secret, the name, website and description are shown to users by the oauth login page.
     * @param name - The name for the application
     * @param website - The URL of the application's website
     * @param description - A description of the application
     * @return Consumer
     * @throws IndiceeAPIConnectorException
     */
    public Consumer createConsumer(String name, String website, String description) throws IndiceeAPIConnectorException;
    
    /**
     * Creates an application, so consumer key and secret, the name, website and description are shown to users by the oauth login page.
     * @param name - The name for the application
     * @param website - The URL of the application's website
     * @param description - A description of the application
     * @param ownerId - The owner for the application - the token needs to be for a user that's an administrator of this user.
     * @return Consumer
     * @throws IndiceeAPIConnectorException
     */
    public Consumer createConsumer(String name, String website, String description, Long ownerId) throws IndiceeAPIConnectorException;

    /**
     * Create a token for a user.
     * @param userEmail - the user to create a token for - the token needs to be for a user that's an administrator of this user.
     * @return Token
     * @throws IndiceeAPIConnectorException
     */
    public Token createToken(String userEmail) throws IndiceeAPIConnectorException;
    
    /**
     * Change the application scope for a user,  the caller token needs to be for a user that's an administrator of this user.
     * @param tokenID - the token to update
     * @param scope - the new scope
     * @return Token
     * @throws IndiceeAPIConnectorException
     */
    public Token updateToken(String tokenID, long scope) throws IndiceeAPIConnectorException;

    /**
     * Create a token for a user of the specified application.
     * @param userEmail - the user to create a token for - the token needs to be for a user that's an administrator of this user.
     * @param consumerKey - the consumer key for the application to create a token for
     * @return Token
     * @throws IndiceeAPIConnectorException
     */
    public Token createToken(String userEmail, String consumerKey) throws IndiceeAPIConnectorException;
    
    /**
     * Delete a user's application token.
     * @param tokenID - the token to delete
     * @param tokenSecret - the secret for the token
     * @throws IndiceeAPIConnectorException
     */
    public void deleteToken(String tokenID, String tokenSecret) throws IndiceeAPIConnectorException;

    /**
     * Delete an application / consumer
     * @param consumerID - the consumer key
     * @param consumerSecret - the consumer secret
     * @throws IndiceeAPIConnectorException
     */
    public void deleteConsumer(String consumerID, String consumerSecret) throws IndiceeAPIConnectorException;
    
    /**
     * Retrieve a consumer by ID, the caller token needs to be an administrator of the owner of the consumer. 
     * @param consumerID
     * @return Consumer 
     * @throws IndiceeAPIConnectorException
     */
    public Consumer getConsumer(String consumerID) throws IndiceeAPIConnectorException;

    /**
     * Retrieve a token by ID, the caller token needs to be the application owner. 
     * @param tokenID
     * @return Token
     * @throws IndiceeAPIConnectorException
     */
    public Token getToken(String tokenID) throws IndiceeAPIConnectorException;

    /**
     * List all the caller token's user's consumers
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public List<Consumer> listConsumers() throws IndiceeAPIConnectorException;

    /**
     * List the tokens for the application, the caller token needs to be the application owner. 
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public List<Token> listTokens() throws IndiceeAPIConnectorException;
}
