package com.indicee.api.services.interfaces;

import java.util.List;

import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.objects.collection.Collection;
import com.indicee.api.objects.collection.CollectionShareScope;
/**
 * The collection service will be used to create, delete, update, show, list, addDataset, deleteDataset, share and unshare
 * Collections from Indicee.
 * 
 */
public interface CollectionService {
	
	/**
	 * Creates a collection
	 * @param Collection - the collection to create
	 * @return Collection
	 * @throws IndiceeAPIConnectorException
	 */
	public Collection createCollection(Collection collection) throws IndiceeAPIConnectorException;
	
	/**
	 * This will delete a collection by id
	 * @param qname
	 * 			collection Id
	 * @throws IndiceeAPIConnectorException
	 */
	public void deleteCollection(long collectionId) throws IndiceeAPIConnectorException;
	
	/**
	 * This will update a collection. It will use name, qname and description of the Collection instance
	 * @param collection
	 * 			collection instance to be updated
	 * @throws IndiceeAPIConnectorException
	 */
	public void updateCollection(Collection collection) throws IndiceeAPIConnectorException;
	
	/**
	 * This will show a collection by its qname
	 * @param qname
	 * 			collection qualified name
	 * @return Collection
	 * @throws IndiceeAPIConnectorException
	 */
	public Collection showCollection(String qname) throws IndiceeAPIConnectorException;
	
	/**
	 * Returns the list of collections
	 * 
	 * @return List of Collection
	 * @throws IndiceeAPIConnectorException
	 */
	public List<Collection> listCollection() throws IndiceeAPIConnectorException;
	
	/**
	 * Returns the list of collection for the given Infospace
	 * 
	 * @param Infospace
	 * @return
	 * @throws IndiceeAPIConnectorException
	 */
	public List<Collection> listCollectionByInfospace(Long Infospace) throws IndiceeAPIConnectorException; 

	/**
	 * This will add a dataset to a collection by their qnames
	 * 
	 * @param collectionQname
	 * 			collection qualified name
	 * @param datasetQname
	 * 			dataset qualified name
	 * @throws IndiceeAPIConnectorException
	 */
	public void addDataset(String collectionQname, String datasetQname) throws IndiceeAPIConnectorException;
	
	/**
	 * This will remove a dataset from a collection, by their qnames
	 * @param collectionQname
	 * 			collection qualified name
	 * @param datasetQname
	 * 			dataset qualified name
	 * @throws IndiceeAPIConnectorException
	 */
	public void deleteDataset(String collectionQname, String datasetQname) throws IndiceeAPIConnectorException;

	
	/**
	 * This will find all the collections shared to a specific group.
	 * @param groupId
	 * @return List of Collection
	 * @throws IndiceeAPIConnectorException
	 */
	public List<Collection> listByGroup(long groupId) throws IndiceeAPIConnectorException ;
	
	/**
	 * This will share a collection with a group given the scope.
	 * @param qname - dataset qualified name
	 * @param scope - the sharing scope for the collection
	 * @param groupId - Id of the group to share to
	 * @throws IndiceeAPIConnectorException
	 */
	public void shareCollection(String qname, long groupId, CollectionShareScope scope) throws IndiceeAPIConnectorException;
	
	/**
	 * Share all the user's collections to the specified group
	 * @param groupId - Id of the group to share to
	 * @param scope - the sharing scope for the collections
	 * @throws IndiceeAPIConnectorException
	 */
	public void shareAllCollections(long groupId, CollectionShareScope scope) throws IndiceeAPIConnectorException;
	
	/**
	 * Share all the user's collections to the specified group
	 * @param groupId - Id of the group to share to
	 * @param scope - the sharing scope for the collections
	 * @param force - share even collections that are already shared, updating their share scope.
	 * @throws IndiceeAPIConnectorException
	 */
	public void shareAllCollections(long groupId, CollectionShareScope scope, boolean force) throws IndiceeAPIConnectorException;
	
	/**
	 * This will unshare a collection from a group
	 * @param qname - the qualified name of the collection
	 * @param scope - the scope to unshare
	 * @param groupId - Id of the group to remove the sharing from
	 * @throws IndiceeAPIConnectorException
	 */
	public void unshareCollection(String qname, long groupId, CollectionShareScope scope) throws IndiceeAPIConnectorException;
	
	/**
	 * This will unshare all the user's collections from a group
	 * @param groupId - Id of the group to remove the sharing from
	 * @param scope - the scope to unshare
	 * @throws IndiceeAPIConnectorException
	 */
	public void unshareAllCollections(long groupId, CollectionShareScope scope) throws IndiceeAPIConnectorException;
}
