package com.indicee.api.services.interfaces;

import java.io.File;
import java.io.OutputStream;
import java.util.List;
import java.util.UUID;

import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.connector.util.InputStreamWrapper;
import com.indicee.api.objects.Page;
import com.indicee.api.objects.contribution.Contribution;

/**
 * The Contribution service is used to upload contributions to Indicee.
 * 
 */
public interface ContributionService {

    /**
     * Delete a dataset's contribution
     * @param datasetName - the name of the dataset
     * @param id - the id of the contribution.
     * @return
     * @throws IndiceeAPIConnectorException
     */
    String delete(String datasetName, long id) throws IndiceeAPIConnectorException;

    /**
     * Get a paged list of contributions to da dataset
     * @param datasetQualifiedName - the qualified name of the dataset
     * @param page - representation of the page within the list of all contributions.
     * @return list of Contribution
     * @throws IndiceeAPIConnectorException
     */
    List<Contribution> list(String datasetQualifiedName, Page page) throws IndiceeAPIConnectorException;
    
    /**
     * List all the contributions for the specified dataset.
     * @param datasetQualifiedName - the qualified name of the dataset
     * @return List of Contribution
     * @throws IndiceeAPIConnectorException
     */
    List<Contribution> list(String datasetQualifiedName) throws IndiceeAPIConnectorException;

    /**
     * Get the latest contribution to a dataset
     * @param datasetQualifiedName
     * @return The most recent Contribution or null if there are none 
     * @throws IndiceeAPIConnectorException
     */
    Contribution latest(String datasetQualifiedName) throws IndiceeAPIConnectorException;
    
    /**
     * This will upload the given file to Indicee. If the contribution matches
     * an existing dataset it will automatically be added to that dataset.
     * 
     * @param file - The contribution file to upload
     * @return The contribution information
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error. This error will also be thrown if
     *             there is an error reading the file.
     */
    public Contribution upload(File file) throws IndiceeAPIConnectorException;

    /**
     * This will upload the given file to Indicee. It will add the contribution
     * to the specified dataset.
     * 
     * @param file - The contribution file to upload
     * @param datasetQualifiedName - The qualified name of the dataset to upload the contribution to.
     * @return The contribution information
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error. This error will also be thrown if
     *             there is an error reading the file.
     */
    public Contribution upload(File file, String datasetQualifiedName) throws IndiceeAPIConnectorException;
    
    /**
     * This will upload given file to Indicee. It will create a new dataset with the specified name and add the
     * contribution to it.
     * If forceSameName is set, if it fails to create a dataset with that name it will response with exception, 
     * otherwise it tries creating a unique name (so the dataset created will have a slightly different name to the one specified). 
     * 
     * @param file
     * @param datasetName
     * @param forceSameName
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public Contribution uploadToNewDataset(File file, String datasetName, Boolean forceSameName) throws IndiceeAPIConnectorException;
    
    /**
     * This will upload given stream to Indicee. It will create a new dataset with the specified name and add the
     * contribution to it.
     * If forceSameName is set, if it fails to create a dataset with that name it will response with exception, 
     * otherwise it tries creating a unique name (so the dataset created will have a slightly different name to the one specified). 
     * 
     * @param file
     * @param datasetName
     * @param forceSameName
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public Contribution uploadToNewDataset(InputStreamWrapper inputStreamWrapper, String datasetName, Boolean forceSameName) throws IndiceeAPIConnectorException;
    
    /**
     * Uploads file to a matching dataset in the specified Infospace
     * @param file
     * @param infospaceId
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public Contribution upload(File file, Long infospaceId) throws IndiceeAPIConnectorException;
    
    /**
     * Uploads file to a new dataset in the specified infospace
     * @param file
     * @param datasetName
     * @param forceSameName
     * @param infospaceId
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public Contribution uploadToNewDataset(File file, String datasetName, Boolean forceSameName, Long infospaceId) throws IndiceeAPIConnectorException;
    
    /**
     * Uploads from inpustreamWrappr to a new dataset in the specified infospace
     * @param inputStreamWrapper
     * @param datasetName
     * @param forceSameName
     * @param infospaceId
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public Contribution uploadToNewDataset(InputStreamWrapper inputStreamWrapper, String datasetName, Boolean forceSameName, Long infospaceId) throws IndiceeAPIConnectorException;
    	
    /**
     * This will upload the given file to Indicee. The contribution will be
     * added to the specified dataset. An error is returned it the signature of
     * the dataset does not match the signature of the contribution
     * THIS IS ONLY SUPPORT IN V1 of the API
     * 
     * @param file
     *            The contribution file to upload
     * @param datasetID
     *            The id of the dataset to add the contribution to.
     * @return The contribution information
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error. This error will also be thrown if
     *             there is an error reading the file.
     */
    @Deprecated
    public Contribution upload(File file, UUID datasetID) throws IndiceeAPIConnectorException;
    
    

    /**
     * This will upload the given stream to Indicee. If the contribution matches
     * existing dataset it will automatically be added to that dataset, otherwise a new dataset 
     * will be created.
     * 
     * @param inputStreamWrapper - Wrapper around an input stream containing the contribution's data
     * @return Contribution
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Contribution upload(InputStreamWrapper inputStreamWrapper) throws IndiceeAPIConnectorException;

    /**
     * This will upload the given file to Indicee. The contribution will be
     * added to the specified dataset. An error is returned it the signature of
     * the dataset does not match the signature of the contribution
     * 
     * @param inputStreamWrapper - Wrapper around an input stream containing the contribution's data
     * @param datasetID - The id of the dataset to add the contribution to.
     * @return Contribution
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Contribution upload(InputStreamWrapper inputStreamWrapper, String datasetID)
            throws IndiceeAPIConnectorException;
    
    /**
	 * This will download the contribution by id
	 * @param contributionId - the id of the contribution
	 * @return contribution in byte array format
	 */
    public byte[] downloadContribution(long contributionId) 
			throws IndiceeAPIConnectorException ;

    /**
     * This will download the contribution by id.
     * @param contributionId - the id of the contribution
     * @param os the data is written to this stream.
     */
    public void downloadContribution(long contributionId, OutputStream os) throws IndiceeAPIConnectorException;
    
	/**
	 * This will get the contribution identified by id
	 * @param contributionId - the id of the contribution
	 * @return Contribution
	 */
	Contribution get(long contributionId) throws IndiceeAPIConnectorException;
	
	/**
	 * This will set the ancillary name, value pair for a specific contribution in a dataset 
	 * 
	 * @param contribitionId - the id of the contribution
	 * 		
	 * @param datasetQname - dataset's qualified name
	 * @param ancillaryName - the name of the ancillary field
	 * @param ancillaryValue - the value for  the ancillary field
	 */
	public void setAncillary(long contribitionId, String datasetQname, String ancillaryName, String ancillaryValue)
			throws IndiceeAPIConnectorException;

	/**
	 * This will delete the ancillary field's value for a specific contribution in a dataset
	 * 
	 * @param contribitionId
	 * @param datasetQname - dataset's qualified name
	 * @param ancillaryName - the name of the ancillary field
	 * @throws IndiceeAPIConnectorException
	 */
	public void deleteAncillary(long contribitionId, String datasetQname, String ancillaryName)
			throws IndiceeAPIConnectorException;

	/**
	 * Load an existing contribution to another dataset that has a matching signature
	 * @param contributionId - the id of the contribution
	 * @param datasetQname - the qualified name of the dataset to load the data into.
	 * @param copyRequired - true to make a copy of the contribution.
	 * @return Contribution
	 * @throws IndiceeAPIConnectorException 
	 */
	public Contribution load(long contributionId, String datasetQname, boolean copyRequired) throws IndiceeAPIConnectorException;

	public JobService getJobService();
}