package com.indicee.api.services.interfaces;

import java.util.List;

import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.objects.dashboard.Dashboard;
import com.indicee.api.objects.dashboard.DashboardMetadata;
import com.indicee.api.objects.dashboard.DashboardShareScope;


/**
 * The dashboard service will be used to list, show, create, delete and update a dashboard in Indicee.
 */
public interface DashboardService {

	/**
	 * This will list all the dashboards for the user. It will return 100 rows (rpp=100) and starting from row 0 (cursor = 0)
	 * 
	 * @return List of Dashboard
	 * @throws IndiceeAPIConnectorException
	 */
	public List<Dashboard> list() throws IndiceeAPIConnectorException ;
	
	/**
	 * This will list all dashboard for the user, given the rows per page, and the starting point (cursor)
	 * @param rpp - records per page, default value = 100
	 * @param cursor - start offset of the dashboard list. default value = 0
	 * @return List of Dashboard
	 * @throws IndiceeAPIConnectorException
	 */
	public List<Dashboard> list(Long rpp, Long cursor) throws IndiceeAPIConnectorException ;
	
	
	/**
	 * List the dashboards in the specified infospace
	 * @param infospaceId - the id of the infospace that contains the dashboards
	 * @param rpp - the number of records per page
	 * @param cursor - the page offset
	 * @return List of Dashboard
	 * @throws IndiceeAPIConnectorException
	 */
	public List<Dashboard> listByInfospace(Long infospaceId, Long rpp, Long cursor) throws IndiceeAPIConnectorException ;
	
	
	/**
	 * List the dashboards in the specified infospace
	 * @param infospaceId - the id of the infospace that contains the dashboards
	 * @return List of Dashboard
	 * @throws IndiceeAPIConnectorException
	 */
	public List<Dashboard> listByInfospace(Long infospaceId) throws IndiceeAPIConnectorException ;
	
	
	/**
	 * List the dashboards share to the specfied group.
	 * @param groupId - the id of the group
	 * @param rpp - the number of records per page
	 * @param cursor - the page offset
	 * @return List of Dashboard
	 * @throws IndiceeAPIConnectorException
	 */
	public List<Dashboard> listByGroup(long groupId, Long rpp, Long cursor) throws IndiceeAPIConnectorException ;
	
	
	/**
	 * List the dashboards share to the specfied group.
	 * @param groupId - the id of the group
	 * @return List of Dashboard
	 * @throws IndiceeAPIConnectorException
	 */
	public List<Dashboard> listByGroup(long groupId) throws IndiceeAPIConnectorException ;

	
	/**
	 * This will return a dashboard given its qname
	 * @param qname - the qualified name of the dashboard
	 * @return Dashboard
	 * @throws IndiceeAPIConnectorException
	 */
	public Dashboard show(String qname) throws IndiceeAPIConnectorException ;
	
	/**
	 * This will create a dashboard from input Dashboard instance.
	 * 
	 * @param dashboard - the dashboard to be created
	 * @return Dashboard
	 * @throws IndiceeAPIConnectorException
	 */
	public Dashboard create(Dashboard dashboard) throws IndiceeAPIConnectorException ;
	
	/**
	 * This will delete a dashboard by its qname
	 * @param qname - the qualified name of the dashboard
	 * @throws IndiceeAPIConnectorException
	 */
	public void delete(String qname) throws IndiceeAPIConnectorException ;
	
	/**
	 * This will update a dashboard by its Id.
	 * 
	 * @param dashboardId - the id of the dashboard to update
	 *
	 * @param dashboard - the dashboard's definition.
	 * @return Dashboard
	 * @throws IndiceeAPIConnectorException
	 */
	public Dashboard update(Long dashboardId, Dashboard dashboard) throws IndiceeAPIConnectorException ;
	
	/**
	 * Share the dashboard to the specified group with the given scope
	 * @param qname - the qualified name of the dashboard.
	 * @param groupId - the group id
	 * @param scope - the scope to share as.
	 * @throws IndiceeAPIConnectorException
	 */
	public void share(String qname, long groupId, DashboardShareScope scope) throws IndiceeAPIConnectorException;
	
	/**
	 * Unshare a dashboard from a group with the given scope
	 * @param qname - the qualified name of the dashboard.
	 * @param groupId - the group id
	 * @param scope - the scope to unshare
	 * @throws IndiceeAPIConnectorException
	 */
	public void unshare(String qname, long groupId, DashboardShareScope scope) throws IndiceeAPIConnectorException;
	
	/**
	 * Share all the dashboards in the current infospace to the specified group with the given scope.
	 * @param groupId - the group id
	 * @param scope - the scope to share as.
	 * @throws IndiceeAPIConnectorException
	 */
	public void shareAll(long groupId, DashboardShareScope scope) throws IndiceeAPIConnectorException;
	
	/**
	 * Share all the dashboards in the current infospace to the specified group.
	 * @param groupId - the group id
	 * @param scope - the scope to share as.
	 * @param force - true to overwrite any existing scoping
	 * @throws IndiceeAPIConnectorException
	 */
	public void shareAll(long groupId, DashboardShareScope scope, boolean force) throws IndiceeAPIConnectorException;
	
	/**
	 * Unshare all the dashboards in the current infospace with the given scope.
	 * @param groupId - the group id
	 * @param scope - the scope to unshare as
	 * @throws IndiceeAPIConnectorException
	 */
	public void unshareAll(long groupId, DashboardShareScope scope)  throws IndiceeAPIConnectorException;
	
	public DashboardMetadata getMetadata(String qname) throws IndiceeAPIConnectorException;
	
	public void setMetadata(DashboardMetadata metadata) throws IndiceeAPIConnectorException;
}
