package com.indicee.api.services.interfaces;

import java.io.File;
import java.util.List;

import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.objects.infospace.Infospace;
/**
 * The interface service will be used to set, deploy, copy, download, list, create, update, delete and show Infospaces from Indicee.
 * 
 */
public interface InfospaceService {
	
	/**
	 * Set the user's active infospace to the one with the specified id.
	 * @param infospaceId - the infospace's id
	 * @throws IndiceeAPIConnectorException
	 */
	public void setInfospace(long infospaceId) throws IndiceeAPIConnectorException;
	
	/**
	 * @param userId
	 * @throws IndiceeAPIConnectorException
	 */
	public void applyDepolyments(long userId) throws IndiceeAPIConnectorException;
	
	/**
	 * Deploy an infospace to a user, the application token must be for a user that's an administrator of the specified user
	 * @param infospaceId - the infospace id
	 * @param userId - the id of the user to deploy to
	 * @param forceUpgrade - for the deployment
	 * @param overwriteData - overwrite the data from the current infospace
	 * @param removeReports - remove reports not in the deployed infospace
	 * @throws IndiceeAPIConnectorException
	 */
	public void deployInfospace(long infospaceId, long userId, boolean forceUpgrade,
								boolean overwriteData, boolean removeReports) throws IndiceeAPIConnectorException;
	
	/**
	 * Copy the contents of one infospace over another
	 * @param infospaceId - the id of the target infospace
	 * @param srcInfospaceId - the id of the source infospace
	 * @param includeData - true to include the datasets' contributions.
	 * @throws IndiceeAPIConnectorException
	 */
	public void copyInfospace(long infospaceId, long srcInfospaceId, boolean includeData) throws IndiceeAPIConnectorException;
	
	/**
	 * Download the infospace (as zipped data) to memory
	 * @param infospaceId - the id of the infospace
	 * @return byte[]
	 * @throws IndiceeAPIConnectorException
	 */
	public byte[] downloadInfospace (long infospaceId) throws IndiceeAPIConnectorException;
	
	/**
	 * Download the infospace (as zipped data) to disk
	 * @param infospaceId - the id of the infospace
	 * @param file - the file to download to
	 * @throws IndiceeAPIConnectorException
	 */
	public void downloadInfospace (long infospaceId, File file) throws IndiceeAPIConnectorException;
	
	/**
	 * Upload an infospace from a file to indicee.
	 * @param infospaceId - the id of the infospace
	 * @param file - the file to upload from
	 * @throws IndiceeAPIConnectorException
	 */
	public void uploadInfospace(long infospaceId, File file) throws IndiceeAPIConnectorException;
	
	/**
	 * List the token user's infospaces
	 * @return List of Infospace
	 * @throws IndiceeAPIConnectorException
	 */
	public List<Infospace> listInfospace() throws IndiceeAPIConnectorException;
	
	/**
	 * Create a new Infospace
	 * @param infospace - the definition (just name, description etc.)
	 * @return Infospace
	 * @throws IndiceeAPIConnectorException
	 */
	public Infospace createInfospace(Infospace infospace) throws IndiceeAPIConnectorException;
	
	/**
	 * Update an infospace's metadata (just name, description etc.)
	 * @param infospaceId - the infospace id
	 * @param infospace - the name, description & version
	 * @return Infospace
	 * @throws IndiceeAPIConnectorException
	 */
	public Infospace updateInfospace(long infospaceId, Infospace infospace) throws IndiceeAPIConnectorException;
	
	/**
	 * Delete an infospace by id
	 * @param infospaceId - the infospace id
	 * @throws IndiceeAPIConnectorException
	 */
	public void deleteInfospace(long infospaceId) throws IndiceeAPIConnectorException;
	
	/**
	 * Get the name, description & version of the currently active infospace
	 * @return Infospace
	 * @throws IndiceeAPIConnectorException
	 */
	public Infospace showCurrentInfospace() throws IndiceeAPIConnectorException;
	
	/**
	 * Get the name, description & version of the specified infospace
	 * @param infospaceId - the id of the infospace
	 * @return Infospace
	 * @throws IndiceeAPIConnectorException
	 */
	public Infospace showInfospace(long infospaceId) throws IndiceeAPIConnectorException;

	/**
	 * Deploy an infospace to a user, the application token must be for a user that's an administrator of the specified user
	 * @param infospaceId - the infospace id
	 * @param userId - the id of the user to deploy to
	 * @param forceUpgrade - for the deployment
	 * @param overwriteData - overwrite the data from the current infospace
	 * @param removeReports - remove reports not in the deployed infospace
	 * @param forceDeployToCurrentInfospace if set true, will force upgrade to current infospace even if the user has attribute "indicee.manage_infospaces" set to true
	 * @throws IndiceeAPIConnectorException
	 */
	void deployInfospace(long infospaceId, long userId, boolean forceUpgrade,
			boolean overwriteData, boolean removeReports,
			Boolean forceDeployToCurrentInfospace)
			throws IndiceeAPIConnectorException;

}
