package com.indicee.api.services.interfaces;

import java.util.List;

import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.objects.model.Attribute;
import com.indicee.api.objects.model.Dataset;
import com.indicee.api.objects.model.Entity;
import com.indicee.api.objects.model.Field;
import com.indicee.api.objects.model.Hierarchy;
import com.indicee.api.objects.model.Measure;
import com.indicee.api.objects.model.Model;
import com.indicee.api.objects.model.Relationship;
import com.indicee.api.objects.model.SourceField;
import com.indicee.api.objects.model.Table;
import com.indicee.api.objects.model.binding.Projection;
import com.indicee.api.objects.model.binding.SourceTableBinding;

/**
 * Used for manipulating datasets, tables, fields, entities, measures and relationships. 
 */
public interface ModelService {

	/**
	 * This will create a SourceTableBinding and return uuid of the new binding
	 * 
	 * @param binding - binds a table's columns to fields
	 * @return uuid of new binding
	 * @throws IndiceeAPIConnectorException    
	 */
    public String createBinding(SourceTableBinding binding) throws IndiceeAPIConnectorException;
    
    /**
     * This will retrieve an existing SourceTableBinding for a table within a dataset
     * @param datasetName
     * @param tableName
     * @return SourceTableBinding
     * @throws IndiceeAPIConnectorException
     */
    public SourceTableBinding getSourceTableBinding(String datasetName, String tableName) throws IndiceeAPIConnectorException; 
    
    /**
     * This will create a Mapping between a field and a projection inside a binding. 
     * The binding is referenced by dataset name and table name
     * 
     * @param datasetName
     * @param tableName
     * @param fieldName
     * @param projectionName
     * @return
     * 		uuid of new mapping
     * @throws IndiceeAPIConnectorException
     */
    public String createBindingMapping(String datasetName, String tableName, String fieldName, String projectionName)
            throws IndiceeAPIConnectorException;
    
	/**
	 * This will delete a Mapping between a field and a projection inside a binding
	 * The binding is referenced by dataset name and table name 
	 * @param datasetName
	 * @param tableName
	 * @param fieldName
	 */
	public String deleteBindingMapping(String datasetName, String tableName, String fieldName) throws IndiceeAPIConnectorException;


    /**    
     * This will create a projection in a binding, given its dataset name and table name.
     * 
     * @param datasetName
     * @param tableName
     * @param projection
     * 		projection should have name, data_type and formula
     * @return
     * 		uuid of new projection
     * @throws IndiceeAPIConnectorException
     */
    public String createBindingProjection(String datasetName, String tableName, Projection projection)
            throws IndiceeAPIConnectorException;

	/**
	 * This will update an existing projection in a binding given its dataset name and table name
	 * @param datasetName
	 * @param tableName
	 * @param projection
	 */
	public String updateBindingProjection(String datasetName, String tableName, Projection projection) throws IndiceeAPIConnectorException;


	/**
	 * This will delete an existing projection in a binding given its dataset name and table name
	 * @param datasetName
	 * @param tableName
	 * @param projection
	 */
	public String deleteBindingProjection(String datasetName, String tableName, String projection) throws IndiceeAPIConnectorException;


    /**
     * This wil create a dataset and return the uuid
     * @param dataset
     * @return
     * 		uuid of new dataset
     * @throws IndiceeAPIConnectorException
     */
    public String createDataset(Dataset dataset) throws IndiceeAPIConnectorException;
    
    /**
     * This will create a dataset in the infospace given and return the uuid
     * @param dataset
     * @param infospaceId the id of the infospace to create the dataset in
     * @return
     * 		uuid of new dataset
     * @throws IndiceeAPIConnectorException
     */
    public String createDataset(long infospaceId, Dataset dataset) throws IndiceeAPIConnectorException;
    
	/**
	 * This will create an attribute in a dataset specified by its name
	 * @param datasetName
	 * @param attribute
	 * 		attribute should have name, type and value
	 * @return
	 * 		true if successful
	 * @throws IndiceeAPIConnectorException
	 */
	public boolean createDatasetAttribute(String datasetName, Attribute attribute) throws IndiceeAPIConnectorException;
    
	/**
	 * This will create an attribute in a dataset specified by its name
	 * @param datasetName
	 * @param attribute
	 * 		attribute should have name, type and value
	 * @param infospaceId
	 * @return
	 * 		true if successful
	 * @throws IndiceeAPIConnectorException
	 */
	public boolean createDatasetAttribute(long infospaceId, String datasetName, Attribute attribute) throws IndiceeAPIConnectorException;
		  
	/**
	 * This will create a table in a dataset sepcified by its name
	 * @param datasetName
	 * @param table
	 * @return
	 * 		uuid of new table
	 * @throws IndiceeAPIConnectorException
	 */
    public String createDatasetTable(String datasetName, Table table) throws IndiceeAPIConnectorException;
    
	/**
	 * This will create a table in a dataset sepcified by its name
	 * @param datasetName
	 * @param infospaceId the infospace id where the dataset is in
	 * @param table
	 * @return
	 * 		uuid of new table
	 * @throws IndiceeAPIConnectorException
	 */
    public String createDatasetTable(long infospaceId, String datasetName, Table table) throws IndiceeAPIConnectorException;
    	  
    /**
     * This will create a source field in a table inside a dataset. 
     * Table and dataset are specified by their names.
     * @param datasetName
     * @param tableName
     * @param sourceField
     * @return
     * 		uuid of new source field
     * @throws IndiceeAPIConnectorException
     */
    public String createDatasetTableField(String datasetName, String tableName, SourceField sourceField)
    		throws IndiceeAPIConnectorException;
    
    /**
     * This will create a source field in a table inside a dataset. 
     * Table and dataset are specified by their names.
     * @param infospaceId
     * @param datasetName
     * @param tableName
     * @param sourceField
     * @return
     * 		uuid of new source field
     * @throws IndiceeAPIConnectorException
     */
    public String createDatasetTableField(long infospaceId, String datasetName, String tableName, SourceField sourceField)
    		throws IndiceeAPIConnectorException;
    
    /**
     * This will create an entity 
     * @param entity
     * @return
     * 		uuid of new entity
     * @throws IndiceeAPIConnectorException
     */
    public String createEntity(Entity entity) throws IndiceeAPIConnectorException;

    /**    
     * This will create an attribute for an entity specified by its name
     * @param entityName
     * @param attribute
     * @return
     * 		uuid of new attribute
     * @throws IndiceeAPIConnectorException
     */
    public String createEntityAttribute(String entityName, Attribute attribute) throws IndiceeAPIConnectorException;

    /**
     * This will create a key with a specified field for an entity.
     * Entity and field are specified by their names
     * @param entityName
     * @param fieldName
     * @return
     * 		uuid of new key
     * @throws IndiceeAPIConnectorException
     */
    public String createEntityKey(String entityName, String fieldName) throws IndiceeAPIConnectorException;
    
    /**
     * This will create a property with specified field for an entity
     * Entity and field are specified by their names
     * @param entityName
     * @param fieldName
     * @return
     * 		uuid of new property
     * @throws IndiceeAPIConnectorException
     */
    public String createEntityProperty(String entityName, String fieldName) throws IndiceeAPIConnectorException;

    /**
     * This will create a field
     * @param table
     * @return
     * 		uuid of new field
     * @throws IndiceeAPIConnectorException
     */
    public String createField(Field table) throws IndiceeAPIConnectorException;
    
    /**
     * This will create a new hierarchy
     * @param hierarchy    
     * @return
     * 		uuid of new hierarchy
     * @throws IndiceeAPIConnectorException
     */
    public String createHierarchy(Hierarchy hierarchy) throws IndiceeAPIConnectorException;
    
    /**
     * This will create a new measure
     * @param measure
     * @return
     * 		uuid of new measure
     * @throws IndiceeAPIConnectorException
     */
    public String createMeasure(Measure measure) throws IndiceeAPIConnectorException;

	/**
	 * Create a relationship between 2 subjects
	 * @param relationship the relationship to create
	 * @throws IndiceeAPIConnectorException 
	 */
    public void createRelationship(Relationship relationship) throws IndiceeAPIConnectorException;
    
    /**
     * This will delete a binding given its dataset name and table name.
     * @param datasetName
     * @param tableName
     * @throws IndiceeAPIConnectorException
     */
    public void deleteBinding(String datasetName, String tableName) throws IndiceeAPIConnectorException;
    
    /**
     * This will delete a dataset given its name
     * @param datasetName    
     * @throws IndiceeAPIConnectorException
     */
    public void deleteDataset(String datasetName) throws IndiceeAPIConnectorException;

    /**
     * This will delete a dataset attribute given the dataset name and attribute name
     * @param datasetName
     * @param attributeName
     * @throws IndiceeAPIConnectorException
     */
    public void deleteDatasetAttribute(String datasetName, String attributeName) throws IndiceeAPIConnectorException;

    /**
     * This will delete a dataset table given its dataset name and table name
     * @param datasetName
     * @param tableName
     * @throws IndiceeAPIConnectorException
     */
    public void deleteDatasetTable(String datasetName, String tableName) throws IndiceeAPIConnectorException;

    /**
     * This will delete entity attribute given the entity name and attribute name
     * @param entityName
     * @param attributeName
     * @throws IndiceeAPIConnectorException
     */
    public void deleteEntityAttribute(String entityName, String attributeName) throws IndiceeAPIConnectorException;
    
    /**
     * This will delete entity given its name 
     * @param entityName
     * @throws IndiceeAPIConnectorException
     */
    public void deleteEntity(String entityName) throws IndiceeAPIConnectorException;
    
    /**
     * This will delete a field give its name
     * @param fieldName
     * @throws IndiceeAPIConnectorException
     */
    public void deleteField(String fieldName) throws IndiceeAPIConnectorException;

    /**
     * This will delete a hierarchy given its name
     * @param hierarchyName
     * @throws IndiceeAPIConnectorException
     */
    public void deleteHierarchy(String hierarchyName) throws IndiceeAPIConnectorException;
    
    /**
     * This will delete a measure given its name
     * @param measureName
     * @throws IndiceeAPIConnectorException
     */
    public void deleteMeasure(String measureName) throws IndiceeAPIConnectorException;
    
    /**
     * This will get a dataset in the user's current infospace given the dataset's name or uuid
     * @param datasetUUIDOrName
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public Dataset getDataset(String datasetUUIDOrName) throws IndiceeAPIConnectorException;
    
    /**
     * This will get a dataset in the specified infospace given the dataset'd name or uuid
     * @param datasetUUIDOrName
     * @param spaceId
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public Dataset getDataset(String datasetUUIDOrNamelong, long spaceId) throws IndiceeAPIConnectorException;

    /**    
     * This will get a dataset table given its dataset name and table name
     * @param datasetName
     * @param tableName
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public Table getDatasetTable(String datasetName, String tableName) throws IndiceeAPIConnectorException;
    
    /**
     * This will return a dataset table source field given its dataset name, table name and source field name
     * @param datasetName
     * @param tableName
     * @param sourceFieldName
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public SourceField getDatasetTableField(String datasetName, String tableName, String sourceFieldName) 
    	throws IndiceeAPIConnectorException;

    /**
     * This will return a list of all entities
     * @return List<Entity>
     * @throws IndiceeAPIConnectorException
     */
    public List<Entity> listEntities() throws IndiceeAPIConnectorException;

    /**
     * This will get an entity given its uuid or name
     * @param entityNameOrUUID
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public Entity getEntity(String entityNameOrUUID) throws IndiceeAPIConnectorException;
    
    
	/**
	 * This will get all the fields defined in the model
	 * @return
	 */
	public List<Field> listFields() throws IndiceeAPIConnectorException;
	
    /** 
     * This will get a field given its uuid or name
     * @param fieldNameOrUUID
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public Field getField(String fieldNameOrUUID) throws IndiceeAPIConnectorException;

    /**
     * This will get a hierarchy given its uuid or name
     * @param hierarchyNameOrUUID
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public Hierarchy getHierarchy(String hierarchyNameOrUUID) throws IndiceeAPIConnectorException;

    
    /**
     * This will get all the measures in the model
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public List<Measure> listMeasures() throws IndiceeAPIConnectorException;
    
    /**
     * This will get a measure given its uuid or name
     * @param measureNameOrUUID
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public Measure getMeasure(String measureNameOrUUID) throws IndiceeAPIConnectorException;
    
    /**
     * This will get the Indicee model and return it as a String.
     * 
     * @return The model XML String
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Model getModel() throws IndiceeAPIConnectorException;
    
    /**
     * This will get the Indicee model and return it as a String.
     * 
     * @param infospaceId
     * @return The model XML String
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Model getModel(Long infospaceId) throws IndiceeAPIConnectorException;

    /**
     * This will list all datasets in the current model.
     * @return
     * 		List of all datasets
     * @throws IndiceeAPIConnectorException
     */
    public List<Dataset> listDatasets() throws IndiceeAPIConnectorException;

    /**
     * This will list all datasets in the current model.
     * @param infospaceId
     * @return
     * 		List of all datasets
     * @throws IndiceeAPIConnectorException
     */
    public List<Dataset> listDatasetsWithInfospace(long infospaceId) throws IndiceeAPIConnectorException;
    
    /**
     * This will list all dataset tables given the datasets name
     * @param datasetName
     * @return
     * 		List of all dataset tables
     * @throws IndiceeAPIConnectorException
     */    
    public List<Table> listDatasetTables(String datasetName) throws IndiceeAPIConnectorException;
    
    /**
     * This will list all hierarchies in the current model
     * @return
     * 		List of all hierarchies
     * @throws IndiceeAPIConnectorException
     */
    public List<Hierarchy> listHierarchies() throws IndiceeAPIConnectorException;

    /**
     * This will list all projections given dataset name and table name
     * @param datasetName
     * @param tableName
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public List<Projection> listProjections(String datasetName, String tableName) throws IndiceeAPIConnectorException;
    
    /**
     * This will set the model to the one provided. Note, make sure the model
     * xml contains a name element.
     * 
     * @param modelString
     *            The model XML
     * @return The new model XML
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public String setModel(String modelString) throws IndiceeAPIConnectorException;

    /**
     * This will update a field
     * @param field
     * @return
     * 		uuid of the field
     * @throws IndiceeAPIConnectorException
     */
    public String updateField(Field field) throws IndiceeAPIConnectorException;
    
    /**
     * This will create or update an attribute of an existing field
     * @param fieldName
     * @param attribute
     */
    public void createFieldAttribute(String fieldName, Attribute attribute) throws IndiceeAPIConnectorException;
    
    /**
     * This will update a dataset with the new description
     * @param uuidOrName
     * 		dataset's name or uuid
     * @param newDescription
     * @return
     * 		uuid of dataset
     * @throws IndiceeAPIConnectorException
     */
    public String updateDataset(String uuidOrName, String newDescription) throws IndiceeAPIConnectorException;

	/**
	 * List the relationships between subjects
	 * @return List<Relationship>
	 * @throws IndiceeAPIConnectorException 
	 */
    public List<Relationship> listRelationships() throws IndiceeAPIConnectorException;

	/**
	 * Delete a relationship between 2 subjects
	 * @param relationship the relationship to delete
	 */
    public void deleteRelationship(Relationship relationship) throws IndiceeAPIConnectorException;
    
    /**
     * This will create a dataset based on an existing template and will create all the bindings/projections for it
     * based on that template's bindings/projections as well.
     * @param template is the dataset with
     * @param newDataset is the dataset used to match signature with
     * @return 
     *		 uuid of the new dataset     
     * @throws IndiceeAPIConnectorException
     */
    public String createDatasetUsingTemplate(Dataset template, Dataset dataset) throws IndiceeAPIConnectorException;
    	
	/**
	 * Will set the Turbo mode for the dataset referenced by qname
	 * 
	 * @param qname - the qualified name of the dataset
	 * @param turnOn - true to turn turbo on, false to turn turbo off
	 * return job id 
	 * @throws IndiceeAPIConnectorException
	 */
	public String setTurboMode(String qname, boolean turnOn) throws IndiceeAPIConnectorException;

	/**
	 * Will return the turbo mode of the dataset references by qname
	 * 
	 * @param qname
	 * @return
	 * @throws IndiceeAPIConnectorException
	 */
	public boolean getTurboMode(String qname) throws IndiceeAPIConnectorException;
	
	
	/**
	 * Create an ancillary field on a dataset, be careful as these cannot be deleted.
	 * The caller needs to make sure that the name they're giving the field is unique across all
	 * the tables in the dataset.
	 * @param datasetName
	 * @param field - requires the name and DataType to be set.
	 * @return The UUID of the source field created
	 * @throws IndiceeAPIConnectorException
	 */
	public String createAncillaryField(String datasetName, SourceField field) throws IndiceeAPIConnectorException;

}
