package com.indicee.api.services.interfaces;

import java.util.List;

import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.objects.Page;
import com.indicee.api.objects.ViewWindow;
import com.indicee.api.objects.dashboard.CachingMetadata;
import com.indicee.api.objects.report.Metadatas;
import com.indicee.api.objects.report.Report;
import com.indicee.api.objects.report.ReportShareScope;
import com.indicee.api.objects.report.RequestStatus;
import com.indicee.api.objects.report.Results;
import com.indicee.api.objects.report.axes.Axes;
import com.indicee.api.objects.report.data.Data;
import com.indicee.api.objects.report.view.View;

/**
 * The report service is use to create, and view reports from Indicee.
 * 
 */
public interface ReportServiceV3 {

    /**
     * Creates a report with the given name. The default {@link ViewWindow} is
     * used.
     * 
     * @param name
     *            The name of the report to use.
     * @return The report ID.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Report createReport(String name) throws IndiceeAPIConnectorException;

    /**
     * Creates a report with the given name and view.
     * 
     * @param name
     *            The name of the report to use.
     * @param view
     *            The view for the report.
     * @return The report ID.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Report createReport(String name, View view) throws IndiceeAPIConnectorException;
    
    /**
     * Deletes the report give the report qname.
     * 
     * @param qname
     *            The qname of the report to delete.
     * @return The report qname.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Report deleteReport(String qname) throws IndiceeAPIConnectorException;


    /**
     * Returns the report give the report id. The dedfault {@link ViewWindow} is
     * used.
     * 
     * @param qname
     *            The qname of the report to use.
     * @return The report specified by the given ID.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Report getReport(String qname) throws IndiceeAPIConnectorException;

    /**
     * Returns the report give the report id
     * 
     * @param qname
     *            The qname of the report to use.
     * @param viewWindow
     *            The view window for the data to return.
     * @return The report specified by the given ID.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Report getReport(String qname, ViewWindow viewWindow) throws IndiceeAPIConnectorException;

    /**
     * Returns the report view given the report id.
     * 
     * @param reportView
     *            The view of the report.
     * @return The report view specified by the given ID.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    @Deprecated
    public Axes getReportAxes(View reportView) throws IndiceeAPIConnectorException;

    /**
     * Returns the report view given the report id.
     * 
     * @param reportView
     *            The view of the report.
     * @return The report view specified by the given ID.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    @Deprecated
    public Axes getReportAxes(View reportView, ViewWindow viewWindow) throws IndiceeAPIConnectorException;
    
    /**
     * Retrieve just the numeric data of the report using the default window. 
     * @param reportView - the report's view definition
     * @return Data 
     * @throws IndiceeAPIConnectorException
     */
    @Deprecated
    public Data getReportData(View reportView) throws IndiceeAPIConnectorException;

    /**
     * Retrieve just the numeric data of the report using the specified window.
     * @param reportView - the report's view definition
     * @param viewWindow - the window of data to return
     * @return
     * @throws IndiceeAPIConnectorException
     */
    @Deprecated
    public Data getReportData(View reportView, ViewWindow viewWindow) throws IndiceeAPIConnectorException;

    /**
     * Retrieve just the metadata of the report view
     * @param reportView
     * @return Metadatas
     * @throws IndiceeAPIConnectorException
     */
    @Deprecated
    public Metadatas getReportMetadata(View reportView) throws IndiceeAPIConnectorException;
    
    /**
     * Returns the list of reports (all reports)
     * 
     * @return The list of reports.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public List<Report> listReports() throws IndiceeAPIConnectorException;
    
    /**
     * Returns the list of reports given the Page.
     * @param page
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public List<Report> listReports(Page page) throws IndiceeAPIConnectorException;
    
    /**
     * Returns the list of reports which belongs to the Infospace
     * 
     * @param infospaceId
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public List<Report> listReportsByInfospace(Long infospaceId) throws IndiceeAPIConnectorException;
    
    /**
     * Returns the list of reports which belongs to the Infospace given the Page
     * 
     * @param infospaceId
     * @param page
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public List<Report> listReportsByInfospace(Long infospaceId, Page page) throws IndiceeAPIConnectorException;
    
    /**
     * Returns the list of reports which belongs to the group
     * 
     * @param groupId
     * @return
     * @throws IndiceeAPIConnectorException
     */
    public List<Report> listReportsByGroup(long groupId) throws IndiceeAPIConnectorException;

    /**
     * Update an existing report's name and view definition 
     * @param qname - the report's qualified name
     * @param name - the name of the report
     * @param view - the report's view definition
     * @return Report
     * @throws IndiceeAPIConnectorException
     */
    public Report updateReport(String qname, String name, View view) throws IndiceeAPIConnectorException;    
    
    /**
     * Updates an existing report's name, view and caching settings 
     * @param qname - the qualified name (required)
     * @param name - the new name for the report (optional)
     * @param view - the new view for the report (optional)
     * @param caching - the new caching settings for the report (optional)
     * @return Report
     * @throws IndiceeAPIConnectorException
     */
    public Report updateReport(String qname, String name, View view, CachingMetadata caching) throws IndiceeAPIConnectorException;

    /**
     * This will Asynchronously request a report. 
     * @param view - the definition of the query to run 
     * @param viewWindow - the window of results requested
     * @param ttl - time to live (minutes)
     * @return request Id  - (pass to requestStatus, requestCancel, requestData, requestResults)
     * @throws IndiceeAPIConnectorException
     */
    public String requestReport(View view, ViewWindow viewWindow, Long ttl) throws IndiceeAPIConnectorException;
    
    /**
     * This will Asynchronously request a report. 
     * @param qname - the qualified name of the report to run
     * @param viewWindow - the window of results requested
     * @param ttl - time to live (minutes)
     * @return request Id  - (pass to requestStatus, requestCancel, requestData, requestResults)
     */
    public String requestReport(String qname, ViewWindow viewWindow, Long ttl) throws IndiceeAPIConnectorException;
    
    /**
     * This will Asynchronously request a report.
     * @param view - the definition of the query to run 
     * @param ttl - time to live (minutes)
     * @return request Id  - (pass to requestStatus, requestCancel, requestData, requestResults)
     * @throws IndiceeAPIConnectorException
     */
    public String requestReport(View view, Long ttl) throws IndiceeAPIConnectorException;
    
    /**
     * This will Asynchronously request a report.
     * @param view - the definition of the query to run 
     * @param viewWindow - the window of results requested
     * @return request Id  - (pass to requestStatus, requestCancel, requestData, requestResults)
     * @throws IndiceeAPIConnectorException
     */
    public String requestReport(View view, ViewWindow viewWindow) throws IndiceeAPIConnectorException;
    
    /**
     * This will Asynchronously request a report.
     * @param view - the definition of the query to run 
     * @return request Id  - (pass to requestStatus, requestCancel, requestData, requestResults)
     * @throws IndiceeAPIConnectorException
     */
    public String requestReport(View view) throws IndiceeAPIConnectorException;
    
    /**
     * Get the status of a running report.
     * @param requestId - the id of the request returned from requestReport
     * @return null if the request is no longer valid. 
     * @throws IndiceeAPIConnectorException
     */
    public RequestStatus requestStatus(String requestId) throws IndiceeAPIConnectorException;
    
    /**
     * Cancel a request to run a report.
     * @param requestId - the id of the request returned from requestReport
     * @throws IndiceeAPIConnectorException
     */
    public void requestCancel(String requestId) throws IndiceeAPIConnectorException;
    
    @Deprecated
    public Data requestData(String requestId) throws IndiceeAPIConnectorException;
    
    /**
     * Get the results from the request to run a report
     * @param requestId - the id of the request returned from requestReport
     * @return Results
     * @throws IndiceeAPIConnectorException
     */
    public Results requestResults(String requestId) throws IndiceeAPIConnectorException;
    
    /**
     * Share all the reports to the specified group using the given scope
     * @param groupId - the id of the group to share to
     * @param scope - the scope with which to share the reports.
     * @throws IndiceeAPIConnectorException
     */
    public void shareAllReports(long groupId, ReportShareScope scope) throws IndiceeAPIConnectorException;
    
    /**
     * Share all the reports to the specified group using the given scope
     * @param groupId - the id of the group to share to
     * @param scope - the scope with which to share the reports.
     * @param force - true to share the reports even if they're already shared.
     * @throws IndiceeAPIConnectorException
     */
    public void shareAllReports(long groupId, ReportShareScope scope, boolean force) throws IndiceeAPIConnectorException;
    
    /**
     * Share an individual report to a group with the given scope
     * @param qName - the qualified name of the report
     * @param groupId - the id of the group to share to
     * @param scope - the scope with which to share the report
     * @throws IndiceeAPIConnectorException
     */
    public void shareReport(String qName, long groupId, ReportShareScope scope) throws IndiceeAPIConnectorException;
    
    /**
     * Unshare a report from the specified group with the given scope
     * @param qName - the qualified name of the report to stop sharing 
     * @param groupId - the group from which to remove the sharing of the report
     * @param scope - the scope to remove the report
     * @throws IndiceeAPIConnectorException
     */
    public void unshareReport(String qName, long groupId, ReportShareScope scope) throws IndiceeAPIConnectorException;

    /**
     * Unshare all reports from the specified group with the given scope.
     * @param groupId - the id of the group from which to remove the reports 
     * @param scope - the scopr to remove from the share.
     * @throws IndiceeAPIConnectorException
     */
    public void unshareAllReports(long groupId, ReportShareScope scope) throws IndiceeAPIConnectorException;

}
