package com.indicee.api.util;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.indicee.api.objects.report.Property;
import com.indicee.api.objects.report.Report;
import com.indicee.api.objects.report.axes.Axis;
import com.indicee.api.objects.report.axes.AxisColumn;
import com.indicee.api.objects.report.data.Data;
import com.indicee.api.objects.report.data.DataRow;

/**
 * This helper is to convert 2d v3 report data into v2 report data
 */
public class DataRowInserter {
	
	/**
	 * Converts the passed report into csv using the supplied writer
	 * @param report - the report (with data) to convert 
	 * @param csvWriter - the writer to use to output the csv
	 * @param ignoreTotals - set to true to not write out aggregate rows.
	 * @param includeHeaderInData - if true, headings are included in the report data, otherwise they're not
	 */
	public static void addDataRows(Report report, boolean ignoreTotals, boolean includeHeaderInData){
		List<Axis> axes = report.getAxes().getAxes();
		if (axes.size() != 2) {
			throw new IllegalStateException("Converting to DataRows requires 2 axes, this report has: " + axes.size());
		}
		
		Axis rowsAxis = axes.get(0);
		List<AxisColumn> headingsForRows = getColumnsForRowsMetadata(rowsAxis);
		
		List<DataRow> rows = new ArrayList<DataRow>();
		DataRow dataRow = new DataRow();
		dataRow.setCells(new ArrayList<String>());
		rows.add(dataRow );
		List<String> currentRow = rows.get(rows.size()-1).getCells();
		for (AxisColumn headingForRows : headingsForRows) {
			currentRow.add(headingForRows.getName());
			
			List<Property> properties = headingForRows.getProperties();
			if (properties != null) {
				for (Property property : properties) {
					currentRow.add(property.getName());
				}
			}
		}
		
		Axis columnsAxis = axes.get(1);
		List<String> headingsForColumns = getColumnsForColumnsMedadata(columnsAxis);
		
		for (String headingForColumns : headingsForColumns) {
			currentRow.add(headingForColumns);
		}
		
		DataRow nextRow = new DataRow();
		nextRow.setCells(new ArrayList<String>());
		rows.add(nextRow);
		
		Data data = report.getReportData();
		List<String> cells = data.getReportDataCells();
		
		processRows(rowsAxis, cells, headingsForRows, headingsForColumns.size(), ignoreTotals, rows);
		
		if (!includeHeaderInData){
			// knock out the headings
			rows.remove(0);
		}
		// knock off the last as it'll be empty
		rows.remove(rows.size()-1);
		data.setReportDataRows(rows);
	}

	/**
	 * Converts the passed report into csv using the supplied writer
	 * @param report - the report (with data) to convert 
	 * @param csvWriter - the writer to use to output the csv
	 * @param ignoreTotals - set to true to not write out aggregate rows.
	 */	
	//keeping this method for compatibility with other places calling this signature
	public static void addDataRows(Report report, boolean ignoreTotals) {
		addDataRows(report, ignoreTotals, false); 		
	}

	private static void processRows(Axis rowsAxis, List<String> cells, List<AxisColumn> headingsForRows, int nMeasures, boolean ignoreTotals, List<DataRow> rows) {
		
		List<AxisColumn> columns = rowsAxis.getColumns();
		int cellOffset = 0;
		processColumns(ignoreTotals, columns, 0, cells, headingsForRows, nMeasures, rows, Collections.<String>emptyList(), cellOffset);
	}

	private static int processColumns(boolean ignoreTotals, List<AxisColumn> columns, int depth,
			List<String> cells, List<AxisColumn> headingsForRows, int nMeasures, List<DataRow> rows, List<String> existingHeadings, int cellOffset) {
		
		for (AxisColumn column : columns) {
			String type = column.getType();
			List<String> columnHeadings = new ArrayList<String>(existingHeadings.size() + 1);
			columnHeadings.addAll(existingHeadings);
			String value = column.getValue();
			columnHeadings.add(value);
			List<Property> properties = column.getProperties();
			if (properties != null) {
				for  (Property property : properties) {
					columnHeadings.add(property.getValue());
				}
			} else {
				if ("AGGREGATION".equals(type)) {
					AxisColumn heading = headingsForRows.get(depth);
					properties = heading.getProperties();
					if (properties != null) {
						for  (int i=0; i < properties.size(); ++i) {
							columnHeadings.add("");
						}
					}
				}
			}
			
			List<AxisColumn> subColumns = column.getSubColumns();
			if (subColumns == null || subColumns.size() == 0) {
				if (ignoreTotals && "AGGREGATION".equals(type)) {
					cellOffset += nMeasures;
				} else {
					List<String> currentRow = rows.get(rows.size()-1).getCells();
					for (String columnHeading : columnHeadings ) {
						currentRow.add(columnHeading);
					}
					
					for (int i=0;i<nMeasures;++i) {
						String cell = cells.get(cellOffset++);
						currentRow.add(cell);
					}

					DataRow nextRow = new DataRow();
					nextRow.setCells(new ArrayList<String>());
					rows.add(nextRow);

				}
			} else {
				cellOffset = processColumns(ignoreTotals, subColumns, depth+1, cells, headingsForRows, nMeasures, rows, columnHeadings, cellOffset);
			}
		}
		return cellOffset;
	}

	private static List<String> getColumnsForColumnsMedadata(Axis columnsAxis) {
		List<AxisColumn> columns = columnsAxis.getColumns();
		List<String> headingsForColumns = new ArrayList<String>(columns.size());
		for (AxisColumn column : columns) {
			headingsForColumns.add(column.getValue());
			List<AxisColumn> subColumns = column.getSubColumns();
			if (subColumns != null && subColumns.size() > 0) {
				throw new IllegalStateException("We cannot deal with hierachical columns data.");
			}
		}

		return headingsForColumns;
	}

	private static List<AxisColumn> getColumnsForRowsMetadata(Axis rowsAxis) {
		List<AxisColumn> headingsForRows = new ArrayList<AxisColumn>();
		getColumnsForRowsMetadata(rowsAxis.getColumns(), headingsForRows, 0);
		return headingsForRows;
	}
	
	private static void getColumnsForRowsMetadata(List<AxisColumn> columns, List<AxisColumn> headingsForRows, int depth) {

		for (AxisColumn column : columns) {
			if (headingsForRows.size() <= depth) {
				headingsForRows.add(column);
			}
			List<AxisColumn> subColumns = column.getSubColumns();
			if (subColumns != null) {
				getColumnsForRowsMetadata(subColumns, headingsForRows, depth+1);
			}
		}
	}
}
