package com.indicee.api.util;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import com.indicee.api.objects.report.view.View;
import com.indicee.api.objects.report.view.attributes.Attribute;
import com.indicee.api.objects.report.view.attributes.AttributesOrigin;
import com.indicee.api.objects.report.view.axes.GroupingAxisLayer;
import com.indicee.api.objects.report.view.axes.Layer;
import com.indicee.api.objects.report.view.axes.LayerQueryGrouping;
import com.indicee.api.objects.report.view.axes.QueryValue;
import com.indicee.api.objects.report.view.axes.QueryValueModel;
import com.indicee.api.objects.report.view.axes.QueryValueReport;
import com.indicee.api.objects.report.view.axes.ValuesAxislayer;
import com.indicee.api.objects.report.view.axes.ViewAxis;
import com.indicee.api.objects.report.view.filter.FieldFilter;
import com.indicee.api.objects.report.view.filter.FieldFilterValue;
import com.indicee.api.objects.report.view.filter.Filter;

/**
 * The ViewBuilder is a class to simplify the construction of views.
 * 
 * @author thaggie
 */
public final class ViewBuilder {

	/**
	 * Builds IQNs from names passed
	 */
	public static final class NameBuilder {
		private final long user;
		private final long space;
		private final String reportUuid;
		
		/**
		 * Use this is you want to include the uuid of an already generated report. 
		 * @param user - the user id 
		 * @param space - the user's infospace id
		 * @param reportUuid - the uuid of the report to create custom measure names relative to
		 */
		public NameBuilder(long user, long space, String reportUuid) {
			super();
			this.user = user;
			this.space = space;
			this.reportUuid = reportUuid;
		}
		
		/**
		 * Build IQNs for a specific user / information space
		 * @param user - the user id 
		 * @param space - the user's infospace id
		 */
		public NameBuilder(long user, long space) {
			this(user, space, UUID.randomUUID().toString());
		}

		/**
		 * Create an entity ( Measure / Subject /Date / ... ) IQN
		 * @param name - the unescaped name of the entity
		 * @return the IQN
		 */
		public String entity(String name) {
			return "iqn.2://indicee.com/" + user + "/" + space + "/dataModel/entities/" + escape(name);
		}
		
		/**
		 * Create a field IQN
		 * @param name - the unescaped name of the field
		 * @return the IQN
		 */
		public String field(String name) {
			return "iqn.2://indicee.com/" + user + "/" + space + "/dataModel/fields/" + escape(name);
		}
		
		/**
		 * Generate a custom measure name for a report
		 * if the reportUuid was passed at construction that is used otherwise a random one is.
		 * @return the IQN
		 */
		public String customMeasureName() {
			return customMeasureName(user, space, reportUuid);
		}
		
		/**
		 * Generate a custom measure name for a report
		 * @param user - the user id 
		 * @param space - the user's infospace id
		 * @return the IQN
		 */
		public static String customMeasureName(long user, long space) {
			return customMeasureName(user, space, null);
		}
		
		/**
		 * Generate a custom measure name for a report
		 * @param user - the user id 
		 * @param space - the user's infospace id
		 * @param reportUuid - the uuid of the report to use (null results in a random one being generated).
		 * @return the IQN
		 */
		public static String customMeasureName(long user, long space, String reportUuid) {
			if (reportUuid == null) {
				reportUuid = UUID.randomUUID().toString();
			}
			String uuid = UUID.randomUUID().toString();
			return "iqn.2://indicee.com/" + user + "/" + space + "/report/" + reportUuid + "/query/entities/" + uuid;
		}
		
		/**
		 * Escape a name ready for inclusion in an IQN 
		 * @param original
		 * @return
		 */
		private static String escape(String original) {
			try {
				String escaped = URLEncoder.encode(original, "UTF-8");
				escaped = escaped.replaceAll("\\+", "%20");
				return escaped;
			} catch (UnsupportedEncodingException e) {
				throw new RuntimeException(e);
			}
		}
	}
	
	/**
	 * Used to build attributes with a chaining call pattern, has helpers for common attributes (such as displayName). 
	 */
	public static final class AttributesBuilder {
		final List<Attribute> attributes;

		/**
		 * Create wrapping an existing list of attributes
		 * @param attributes
		 */
		public AttributesBuilder(List<Attribute> attributes) {
			super();
			this.attributes = attributes;
		}
		
		/**
		 * Adds an attribute to the list of attributes, if one is already there it's updated
		 * @param name - the name of the attribute - Eg. displayName
		 * @param type - the type of the attributes - Eg. string
		 * @param value - the value to set for the attribute.
		 * @return AttributesBuilder - for chaining
		 */
		public AttributesBuilder add(String name, String type, String value) {
			Attribute attribute = findOrCreate(name, type);
			attribute.setValue(value);
			return this;
		}

		/**
		 * Removes the named attribute from the list (if it doesn't exist it's a NOP).
		 * @param name
		 * @return AttributesBuilder - for chaining
		 */
		public AttributesBuilder remove(String name) {
			Attribute attribute = find(name);
			if (attribute != null) {
				attributes.remove(attribute);
			}
			return this;
		}

		/**
		 * Helper for setting the 'displayName' attribute
		 * @param value - the value to set
		 * @return AttributesBuilder - for chaining
		 */
		public AttributesBuilder displayName(String value) {
			return add("displayName", "string", value);
		}
		
		/**
		 * Helper for setting the 'formatting' attribute
		 * @param value - the value to set
		 * @return AttributesBuilder - for chaining
		 */
		public AttributesBuilder formatting(String value) {
			return add("formatting", "string", value);
		}
		
		/**
		 * Helper for setting the 'nullDisplayValue' attribute
		 * @param value - the value to set
		 * @return AttributesBuilder - for chaining
		 */
		public AttributesBuilder nullDisplayValue(String value) {
			return add("nullDisplayValue", "string", value);
		}
		
		/**
		 * Find an attribute by name
		 * @param name
		 * @return null if it's not found
		 */
		private Attribute find(String name) {
			if (name == null) {
				throw new IllegalArgumentException("Can't find attribute by name when name given is null.");
			}
			for (Attribute attribute : attributes) {
				if (name.equals(attribute.getName())) {
					return attribute;
				}
			}
			
			return null;
		}
		
		/**
		 * Find an attribute by name - if it doesn't already exist create and add it.
		 * @param name
		 * @param type
		 * @return Attribute
		 */
		private Attribute findOrCreate(String name, String type) {
			Attribute attribute = find(name);
			if (attribute == null) {
				attribute = new Attribute(name, type, (String)null);
				attributes.add(attribute);
			}
			return attribute;
		}
		
	}
	
	private final NameBuilder nb;
	private final View view;
	
	/**
	 * Create a ViewBuilder wrapping an existing view. 
	 * @param nb - the name builder to use when generating IQNs - if null you need to pass IQNs in the calls rather than names.
	 * @param view - the view for the builder to modify
	 */
	public ViewBuilder(NameBuilder nb, View view) {
		if (view == null) {
			throw new IllegalArgumentException("view cannot be null.");
		}
		this.nb = nb;
		this.view = view;
	}

	/**
	 * Create a ViewBuilder that generates IQNs for the given user id / information space
	 * @param user - the user id 
	 * @param space - the user's infospace id
	 * @param view - the view for the builder to modify
	 */
	public ViewBuilder(long user, long space, View view) {
		this(new NameBuilder(user, space), view);
	}
	
	/**
	 * Create a ViewBuilder that the specified number of axes where you pass the full IQNs when adding things.
	 * @param nAxes
	 */
	public ViewBuilder(int nAxes) {
		this(null, createView(nAxes));
	}
	
	/**
	 * Create a ViewBuilder with the specified number of axes that generates IQNs for the given user id / information space
	 * @param user - the user id 
	 * @param space - the user's infospace id
	 * @param nAxes - the number of axes you want in the view.
	 */
	public ViewBuilder(long user, long space, int nAxes) {
		this(new NameBuilder(user, space), createView(nAxes));
	}

	/**
	 * Create a ViewBuilder with the named axes that generates IQNs for the given user id / information space
	 * @param user - the user id 
	 * @param space - the user's infospace id
	 * @param axisNames - the names to give the axes in the view.
	 */
	public ViewBuilder(long user, long space, String... axisNames) {
		this(new NameBuilder(user, space), createView(axisNames));
	}

	/**
	 * Create a ViewBuilder with the named axes where you have to pass fully qualified IQNs when adding things.
	 * @param axisNames - the names to give the axes in the view.
	 */
	public ViewBuilder(String... axisNames) {
		this(null, createView(axisNames));
	}
	
	/**
	 * Getter for the view the build is updating.
	 * @return View
	 */
	public View getView() {
		return view;
	}

	/**
	 * Add a measure to the view
	 * @param axisIndex - usually 1 for the columns
	 * @param entity
	 * @return AttributesBuilder for adding attributes to the measure
	 */
	public AttributesBuilder addMeasure(int axisIndex, String entity) {
		QueryValueModel value = new QueryValueModel();
		value.setEntity(nb == null ? entity : nb.entity(entity));
		
		return addValueAxisQueryValue(axisIndex, value);
	}

	/**
	 * Add a custom measure to the view
	 * @param axisIndex - usually 1 for the columns
	 * @param formula - the formula used to calculate the custom measure
	 * @return AttributesBuilder for adding attributes to the measure
	 */
	public AttributesBuilder addFormula(int axisIndex, String formula) {
		QueryValueReport value = new QueryValueReport();
		value.setName(nb == null ? NameBuilder.customMeasureName(0, 0) : nb.customMeasureName());
		value.setFormula(formula);
		
		return addValueAxisQueryValue(axisIndex, value);
	}

	/**
	 * Add a subject or date to an axis 
	 * @param axisIndex - usually 0 for the rows axis
	 * @param entity
	 * @param properties
	 */
	public void addSubject(int axisIndex, String entity, String... properties) {
		String entityQName = nb == null ? entity : nb.entity(entity); 
		GroupingAxisLayer layer = new GroupingAxisLayer();
		layer.setHierarchy(entityQName);
		LayerQueryGrouping grouping = new LayerQueryGrouping();
		grouping.setEntity(entityQName);

		List<QueryValue> queryValues = new ArrayList<QueryValue>(properties.length);
		for (String property : properties) {
			String propertyQName = nb == null ? property : nb.field(property);
			QueryValue queryValue = new QueryValueModel(entityQName, propertyQName);
			queryValues.add(queryValue);
		}
		grouping.setProperties(queryValues);
		layer.setQueryGrouping(grouping);

		List<ViewAxis> axes = view.getReportViewAxes();
		ViewAxis viewAxis = axes.get(axisIndex);
		List<Layer> layers = viewAxis.getLayers();
		if (layers == null) {
			layers = new ArrayList<Layer>();
			viewAxis.setLayers(layers);
		}
		layers.add(layer);
	}

	/**
	 * Add an inclusive (so filtering to just the values specified) filter to the view
	 * @param entity - the entity to filter
	 * @param field - the field to filter
	 * @param values - the values to limit to
	 */
	public void include(String entity, String field, List<String> values) {
		addFilter(entity, field, true, values);
	}

	/**
	 * Add an inclusive (so filtering to just the values specified) filter to the view
	 * @param entity - the entity to filter
	 * @param field - the field to filter
	 * @param values - the values to limit to
	 */
	public void include(String entity, String field, String... values) {
		addFilter(entity, field, true, Arrays.asList(values));
	}

	/**
	 * Add an excluding (so filtering to the values that are not specified) filter to the view
	 * @param entity - the entity to filter
	 * @param field - the field to filter
	 * @param values - the values to remove
	 */
	public void exclude(String entity, String field, List<String> values) {
		addFilter(entity, field, false, values);
	}

	/**
	 * Add an excluding (so filtering to the values that are not specified) filter to the view
	 * @param entity - the entity to filter
	 * @param field - the field to filter
	 * @param values - the values to remove
	 */
	public void exclude(String entity, String field, String... values) {
		addFilter(entity, field, false, Arrays.asList(values));
	}

	private void addFilter(String entity, String field, boolean include, List<String> values) {
		String entityQName = nb == null ? entity : nb.entity(entity); 
		String fieldQName = nb == null ? field : nb.field(field); 
		FieldFilter fieldFilter = new FieldFilter();
		
		fieldFilter.setEntity(entityQName);
		fieldFilter.setField(fieldQName);

		List<FieldFilterValue> fieldFilterValues = new ArrayList<FieldFilterValue>(values.size());
		for (String value : values) {
			StringBuilder sb = new StringBuilder(value.length() + 2);
			sb.append("\"");
			sb.append(value);
			sb.append("\"");
			FieldFilterValue fieldFilterValue = new FieldFilterValue(sb.toString());
			fieldFilterValues.add(fieldFilterValue);
		}
		fieldFilter.setValues(fieldFilterValues);
		fieldFilter.setInclusive(include);

		List<Filter> fieldFilters = view.getFieldFilters();
		if (fieldFilters == null) {
			fieldFilters = new ArrayList<Filter>();
			view.setFieldFilters(fieldFilters);
		}

		fieldFilters.add(fieldFilter);
	}

	private ValuesAxislayer getValuesAxislayer(int axisIndex) {
		List<ViewAxis> axes = view.getReportViewAxes();
		ViewAxis viewAxis = axes.get(axisIndex);
		List<Layer> layers = viewAxis.getLayers();
		if (layers == null) {
			layers = new ArrayList<Layer>();
			viewAxis.setLayers(layers);
		}
		ValuesAxislayer valuesAxisLayer = null;
		for (Layer layer : layers) {
			if (layer instanceof ValuesAxislayer) {
				valuesAxisLayer = (ValuesAxislayer) layer;
			}
		}
		if (valuesAxisLayer == null) {
			valuesAxisLayer = new ValuesAxislayer();
			layers.add(valuesAxisLayer);
		}
		return valuesAxisLayer;
	}
	
	private List<QueryValue> getValueAxisValues(int axisIndex) {
		ValuesAxislayer valuesAxisLayer = getValuesAxislayer(axisIndex);
		List<QueryValue> queryValues = valuesAxisLayer.getValues();
		if (queryValues == null) {
			queryValues = new ArrayList<QueryValue>();
			valuesAxisLayer.setValues(queryValues);
		}
		return queryValues;
	}


	private static View createView(String[] axisNames) {
		View view = new View();
		List<ViewAxis> reportViewAxes = new ArrayList<ViewAxis>(axisNames.length);
		for (String axisName : axisNames) {
			ViewAxis viewAxis = new ViewAxis();
			viewAxis.setName(axisName);
			reportViewAxes.add(viewAxis);
		}
		view.setReportViewAxes(reportViewAxes);
		return view;
	}

	private static View createView(int nAxes) {
		View view = new View();
		List<ViewAxis> reportViewAxes = new ArrayList<ViewAxis>(nAxes);
		for (int i = 0; i < nAxes; ++i) {
			ViewAxis viewAxis = new ViewAxis();
			viewAxis.setName("Unnamed");
			reportViewAxes.add(viewAxis);
		}
		view.setReportViewAxes(reportViewAxes);
		return view;
	}


	private AttributesBuilder addValueAxisQueryValue(int axisIndex,
			QueryValue value) {
		List<QueryValue> queryValues = getValueAxisValues(axisIndex);
		queryValues.add(value);
		
		return createAttributesBuilder(value);
	}
	
	private AttributesBuilder createAttributesBuilder(QueryValue queryValue) {
		List<AttributesOrigin> originAttributes = queryValue.getAttributes();
		if (originAttributes == null) {
			originAttributes = new ArrayList<AttributesOrigin>(1);
			queryValue.setAttributes(originAttributes);
		}
		
		return createAttributesBuilder(originAttributes);
	}

	private AttributesBuilder createAttributesBuilder(List<AttributesOrigin> originAttributes) {
		// find the default origin attribute
		AttributesOrigin defaultOriginAttribute = null;
		for (AttributesOrigin originAttribute : originAttributes) {
			if ("default".equals(originAttribute.getName())) {
				defaultOriginAttribute = originAttribute;
				break;
			}
		}
		
		if (defaultOriginAttribute == null) {
			List<Attribute> attributes = new ArrayList<Attribute>();
			defaultOriginAttribute = new AttributesOrigin("default", attributes);
			originAttributes.add(defaultOriginAttribute);
		}

		List<Attribute> attributes = defaultOriginAttribute.getReportAttributes();
		return new AttributesBuilder(attributes);
	}
}