package com.indicee.api.util;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Helper class for processing XML
 * @author thaggie
 */
public class XmlHelper {

	private static final String UTF8 = "UTF-8";
	private static final String YES = "yes";
	private static final String INDENT_AMOUNT = "{http://xml.apache.org/xslt}indent-amount";

	private static final ThreadLocal<DocumentBuilderFactory> documentBuilderFactory = new ThreadLocal<DocumentBuilderFactory>() {
		protected DocumentBuilderFactory initialValue() {
			return DocumentBuilderFactory.newInstance();
		};
	};
	
	private static final ThreadLocal<TransformerFactory> transformerFactory = new ThreadLocal<TransformerFactory>() {
		protected TransformerFactory initialValue() {
			return TransformerFactory.newInstance();
		};
	};
	
	private static final ThreadLocal<XPathFactory> xPathFactory = new ThreadLocal<XPathFactory>() {
		protected XPathFactory initialValue() {
			return XPathFactory.newInstance();
		};
	};

	private static final ThreadLocal<XPath> xPath = new ThreadLocal<XPath>() {
		protected XPath initialValue() {
			return xPathFactory.get().newXPath();
		};
	};
	
	/**
	 * This class should just be used via static methods, hence the constructor is hidden as private
	 */
	private XmlHelper() {
	}
	
	/**
	 * Get the child elements of the specified element that match the tags passed in.
	 * @param element - the element who's children we wish to search
	 * @param tags - the tags to match (empty returns all child elements).
	 * @return The child elements that matched
	 */
	public static List<Element> childElements(Element element, String... tags) {
		NodeList nodeList = element.getChildNodes();
		return getElements(nodeList, tags);
	}

	/**
	 * Get the elements who's tags match those passed in. 
	 * @param nodeList - the node list in which to look for elements 
	 * @param tags - the tags to match (empty returns all elements).
	 * @return The elements that matched.
	 */
	public static List<Element> getElements(NodeList nodeList, String... tags) {
		int nNodes = nodeList.getLength();
		List<Element> elements = new ArrayList<Element>(nNodes);
		
		Set<String> tagSet = new HashSet<String>(tags.length);
		for (String tag : tags) {
			tagSet.add(tag);
		}
		for (int i=0; i<nNodes; ++i) {
			Node node = nodeList.item(i);
			if (node instanceof Element) {
				Element element = (Element)node;
				String tagName = element.getTagName();
				if (tagSet.isEmpty() || tagSet.contains(tagName)) {
					elements.add(element);
				}
			}
		}
		return elements;
	}

	/**
	 * Get the first child element of an element. 
	 * @param element - the element whose child we want
	 * @return the child or null if there are no child elements.
	 */
	public static Element child(Element element) {
		List<Element> children = childElements(element);
		return children.size() > 0 ? children.get(0) : null; 
	}
	
	/**
	 * Get the first child of the element by tag name. 
	 * @param element - the element whose children to look at.
	 * @param tag - the tag name for the child
	 * @return the child element or null if there was no child of that tag name
	 */
	public static Element child(Element element, String tag) {
		for (Element child : childElements(element, tag)) {
			String childTag = child.getTagName();
			if (tag.equals(childTag)) {
				return child;
			}
		}
		return null;
	}

	/**
	 * Return the text content of the first child of the element with the specified tag
	 * @param element - the element under which to look for the child element
	 * @param tag - the tag name for the child
	 * @return The text content (or null if there was no child of that name).
	 */
	public static String childText(Element element, String tag) {
		Element child = child(element, tag);
		return child != null ? child.getTextContent() : null;
	}

	/**
	 * Creates an element with the specified tag and appends it to the element supplied
	 * @param element - the element to add the created element to
	 * @param tag - the tag name for the element to create
	 * @return The element created
	 */
	public static Element appendElement(Element element, String tag) {
		Document document = element.getOwnerDocument();
		Element childElement = document.createElement(tag);
		element.appendChild(childElement);
		return childElement;
	}
	
	/**
	 * Creates an element with the specified tag with the text as its content and appends it to the element supplied
	 * @param element - the element to add the created element to
	 * @param tag - the tag name for the element to create
	 * @param text - the text to have as the content of the created Element
	 * @return The element created
	 */
	public static Element appendTextElement(Element element, String tag, String text) {
		Element childElement = appendElement(element, tag);
		childElement.setTextContent(text);
		return childElement;
	}

	/**
	 * Creates an element with the specified tag with the text as the boolean value
	 * @param element - the element to add the created element to
	 * @param tag - the tag name for the element to create
	 * @param value - the value to have as the content of the created Element
	 * @return The element created
	 */
	public static Element appendTextElement(Element element, String tag, boolean value) {
		return appendTextElement(element, tag, Boolean.toString(value));
	}

	/**
	 * Create an empty XML document for building up a DOM tree.
	 * @return the document
	 */
	public static Document createEmptyDocument() {
		try {
			DocumentBuilder documentBuilder = documentBuilderFactory.get().newDocumentBuilder();
			Document document = documentBuilder.newDocument();
			return document;
		} catch (ParserConfigurationException e) {
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * Creates a document that has a deep copy of the element passed in as its document element
	 * @param element
	 * @return the new document
	 */
	public static Document createDocumentFrom(Element element) {
		Document document = createEmptyDocument();
		Node clonedNode = element.cloneNode(true);
		clonedNode = document.adoptNode(clonedNode);
		document.appendChild(clonedNode);
		return document;
	}

	/**
	 * Read an input stream in as an XML document.
	 * @param inputStream
	 * @return the XML document
	 */
	public static Document readXmlDocumentFromStream(InputStream inputStream) {
		try {
			DocumentBuilder documentBuilder = documentBuilderFactory.get().newDocumentBuilder();
			return documentBuilder.parse(inputStream);
		} catch (ParserConfigurationException e) {
			throw new RuntimeException(e);
		} catch (SAXException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * Read a file as an XML document.
	 * @param file
	 * @return the XML document
	 */
	public static Document readXmlDocumentFromFile(File file) {
		try {
			InputStream inputStream = new FileInputStream(file);
			try {
				return readXmlDocumentFromStream(inputStream);
			} finally {
				inputStream.close();
			}
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * Read an input stream in as an XML document.
	 * @param xmlString
	 * @return the XML document
	 */
	public static Document readXmlDocumentFromString(String xmlString) {
		try {
			DocumentBuilder documentBuilder = documentBuilderFactory.get().newDocumentBuilder();
			return documentBuilder.parse(new InputSource(new StringReader(xmlString)));
		} catch (ParserConfigurationException e) {
			throw new RuntimeException(e);
		} catch (SAXException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Write an XML document out to an output stream in UTF-8
	 * @param document - the document to write.
	 * @param outputStream - the stream to which the XML will be written
	 * @param indent - the indent level.
	 */
	public static void writeXmlDocumentToStream(Document document,
			OutputStream outputStream, int indent) {

		try {
			Transformer transformer = transformerFactory.get().newTransformer();
			transformer.setOutputProperty(OutputKeys.ENCODING, UTF8);
			if (indent > 0) {
				transformer.setOutputProperty(OutputKeys.INDENT, YES);
				transformer.setOutputProperty(INDENT_AMOUNT, Integer.toString(indent));
			}
			Result result = new StreamResult(outputStream);
			Source source = new DOMSource(document);
			transformer.transform(source, result);
		} catch (TransformerException e) {
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * Write an XML document out to an output stream in UTF-8
	 * @param document - the document to write.
	 * @param fileName - the file to which the XML will be written
	 * @param indent - the indent level.
	 */
	public static void writeXmlDocumentToFile(Document document,
			String fileName, int indent) {
		File file = new File(fileName);
		writeXmlDocumentToFile(document, file, indent);
	}
	
	/**
	 * Write an XML document out to a file in UTF-8
	 * @param document - the document to write.
	 * @param file - the file to which the XML will be written
	 * @param indent - the indent level.
	 */
	public static void writeXmlDocumentToFile(Document document,
			File file, int indent) {

		try {
			FileOutputStream fos = new FileOutputStream(file);
			try {
				writeXmlDocumentToStream(document, fos, indent);
			} finally {
				fos.close();
			}
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * Write an XML document out to a byte array in UTF-8
	 * @param document - the document to write.
	 * @param indent - the indent level.
	 * @return The XML as a byte array
	 */
	public static byte[] writeXmlDocumentToByteArray(Document document, int indent) {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		writeXmlDocumentToStream(document, baos, indent);
		return baos.toByteArray();
	}
	
	/**
	 * Write an XML document out to a String
	 * @param document - the document to write.
	 * @param indent - the indent level.
	 * @return The XML string
	 */
	public static String writeXmlDocumentToString(Document document, int indent) {
		byte[] bytes = writeXmlDocumentToByteArray(document, indent);
		try {
			return new String(bytes, UTF8);
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException(e);
		}
	}
	
	
	/**
	 * Create an x-path expression from a string
	 * @param exprString
	 * @return XPathExpression - remember folks, these are not thread safe
	 */
	public static XPathExpression xpath(String exprString) {
		try {
			XPathExpression expr = xPath.get().compile(exprString);
			return expr;
		} catch (XPathExpressionException e) {
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * Retrieve node text from a document using an x-path expression
	 * @param document
	 * @param expr
	 * @return The node text content
	 */
	public static String xpathText(Document document, XPathExpression expr) {
		try {
			Object result = expr.evaluate(document, XPathConstants.STRING);
			return (String)result;
		} catch (XPathExpressionException e) {
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * Retrieve node text from a document using an x-path expression string
	 * @param document
	 * @param exprString
	 * @return The node text content
	 */
	public static String xpathText(Document document, String exprString) {
		XPathExpression expr = xpath(exprString);
		return xpathText(document, expr);
	}
	
	public static Element createErrorXml(Document document, Throwable t) {
		Element element = createErrorXml(document, t, "exception");
		element.setAttribute("when", new Date().toString());
		Element systemElement = document.createElement("system");
		element.appendChild(systemElement);
		
		for (Entry<Object, Object> entry : System.getProperties().entrySet()) {
			Element propertyElement = document.createElement("property");
			systemElement.appendChild(propertyElement);
			Object property = entry.getKey();
			propertyElement.setAttribute("name", String.valueOf(property));
			Object value = entry.getValue();
			if (value != null) {
				propertyElement.setAttribute("value", String.valueOf(value));
			}
		}
		
		return element;
	}
	
	private static Element createErrorXml(Document document, Throwable throwable, String tag) {
		
		Element exceptionElement = document.createElement(tag);
		String className = throwable.getClass().getName();
		exceptionElement.setAttribute("class", className);
		
		Element messageElement = document.createElement("message");
		exceptionElement.appendChild(messageElement);
		
		String message = throwable.getMessage();
		try {
			Node exceptionMessage = XmlHelper.readXmlDocumentFromString(message).getDocumentElement();
			exceptionMessage = document.importNode(exceptionMessage, true);
			messageElement.appendChild(exceptionMessage);
		} catch (Throwable t) {
			messageElement.setTextContent(message);
		}
		StackTraceElement[] st = throwable.getStackTrace();
		if (st != null) {
			Element stackElement = document.createElement("stack");
			for (StackTraceElement ste : st) {
				Element frameElement = document.createElement("frame");
				stackElement.appendChild(frameElement);
				frameElement.setTextContent(ste.toString());
			}

			exceptionElement.appendChild(stackElement);
		}
		
		Throwable cause = throwable.getCause();
		if (cause != null && cause != throwable) {
			Element causeElement = createErrorXml(document, cause, "cause");
			exceptionElement.appendChild(causeElement);
		}
		return exceptionElement;
	}
}
