package com.indicee.api.parser;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import junit.framework.TestCase;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.indicee.api.objects.report.Property;
import com.indicee.api.objects.report.Report;
import com.indicee.api.objects.report.axes.AxisColumn;
import com.indicee.api.objects.report.view.View;
import com.indicee.api.objects.report.view.attributes.Attribute;
import com.indicee.api.objects.report.view.attributes.AttributesOrigin;
import com.indicee.api.objects.report.view.filter.EntityKeysFilter;
import com.indicee.api.objects.report.view.filter.FieldValue;
import com.indicee.api.objects.report.view.filter.Filter;
import com.indicee.api.objects.report.view.filter.RelativeTimePeriod;
import com.indicee.api.objects.report.view.filter.RelativeTimePeriodFilter;
import com.indicee.api.util.XmlHelper;

public class IndiceeParserTest extends TestCase {

	public void testParseDate() {
		String xml = "<test>2010-10-21T15:44:40-07:00</test>";//2008-08-24T23:38:00ZZ
		Document document = IndiceeParser.getDocumentFromString(xml);
		Element element = document.getDocumentElement();
		Date date = IndiceeParser.getDate(element);
		System.out.println(date);
		//2010-10-21T15:44:40-07:00
	}
	
	public void testReportParser() {
		File reportFile = new File("samples/report.xml");
		Document reportDocument = XmlHelper.readXmlDocumentFromFile(reportFile);
		
		Report report = ReportParser.createReport(reportDocument);
		View view = report.getReportView();
		List<Filter> filters = view.getFieldFilters();
		assertEquals(1, filters.size());
		
		Filter filter = filters.get(0);
		assertSame(EntityKeysFilter.class, filter.getClass());
		
		EntityKeysFilter keysFilter = (EntityKeysFilter)filter;
		
		ArrayList<ArrayList<FieldValue>> keyValueSets = keysFilter.getKeyValueSets();
		assertEquals(3, keyValueSets.size());
		
		List<AttributesOrigin> originAttributesList = keysFilter.getAttributeOrigins();
		assertEquals(1, originAttributesList.size());
		
		AttributesOrigin originAttributes = originAttributesList.get(0);
		List<Attribute> attributes = originAttributes.getReportAttributes();
		assertEquals(4, attributes.size());
		
		Attribute attribute = attributes.get(1);
		assertEquals("stringList", attribute.getType());
		
		ArrayList<String> valueList = attribute.getValueList();
		assertNotNull(valueList);
		
		assertEquals(6, valueList.size());

		// if we reparse the report and re-write it out to we get the same XML?
		String xmlString = report.toFormatedString();
		Document reportDocument2 = XmlHelper.readXmlDocumentFromString(xmlString);
		Report report2 = ReportParser.createReport(reportDocument2);
		assertEquals(xmlString, report2.toFormatedString());
	}
	
	
	public void testRelativeTimePeriodFilter() {
		String xmlString = 
	    "<relativetimeperiodfilter entity=\"iqn.2://indicee.com/6/0/dataModel/entities/Date\" field=\"iqn.2://indicee.com/6/0/dataModel/fields/Date\">"+
	    "  <relativetimeperiods>"+
	    "    <relativetimeperiod rangeend=\"-1\" rangestart=\"-6\" timeperiod=\"MonthPeriod\"/>"+
	    "  </relativetimeperiods>"+
	    "  <attributes>"+
	    "    <origin name=\"default\">"+
	    "      <attribute name=\"assignedToValueOf\" type=\"string\" value=\"iqn.2://indicee.com/6/0/report/fc001f78-6ebf-45fc-8266-6c3371dba786/parameters/wink\"/>"+
	    "      <attribute name=\"userModified\" type=\"boolean\" value=\"true\"/>"+
	    "    </origin>"+
	    "  </attributes>"+
	    "</relativetimeperiodfilter>";
	  
		Document document = XmlHelper.readXmlDocumentFromString(xmlString);
		Element element = document.getDocumentElement();
		RelativeTimePeriodFilter filter = ReportParser.getRelativeTimePeriodFilter(element);
		
		String entity = filter.getEntity();
		assertEquals("iqn.2://indicee.com/6/0/dataModel/entities/Date", entity);
		
		String field = filter.getField();
		assertEquals("iqn.2://indicee.com/6/0/dataModel/fields/Date", field);
		
		List<RelativeTimePeriod> periods = filter.getRelativeTimePeriods();
		assertEquals(1, periods.size());
		
		RelativeTimePeriod period = periods.get(0);
		assertEquals(-6, period.getRangeStart());
		assertEquals(-1, period.getRangeEnd());
		assertEquals("MonthPeriod", period.getTimePeriod());
		
		List<AttributesOrigin> attributeOrigins = filter.getAttributeOrigins();
		assertEquals(1, attributeOrigins.size());
		
		AttributesOrigin attribute = attributeOrigins.get(0);
		assertEquals("default", attribute.getName());

		List<Attribute> attributes = attribute.getReportAttributes();
		assertEquals(2, attributes.size());
		
		Attribute attribute1 = attributes.get(0);
		assertEquals("assignedToValueOf", attribute1.getName());
		assertEquals("string", attribute1.getType());
		assertEquals("iqn.2://indicee.com/6/0/report/fc001f78-6ebf-45fc-8266-6c3371dba786/parameters/wink", attribute1.getValue());
		
		Attribute attribute2 = attributes.get(1);
		assertEquals("userModified", attribute2.getName());
		assertEquals("boolean", attribute2.getType());
		assertEquals("true", attribute2.getValue());
		
	}
	
	public void testAxisColumnWithProperties() {

		String xmlString = 
        "<column type=\"GROUPING\">"+
        "<name>Job</name>"+
        "<value>740</value>"+
        "<sub>"+
          "<column type=\"GROUPING\">"+
            "<name>Project Manager</name>"+
            "<value>���</value>"+
          "</column>"+
          "<column type=\"AGGREGATION\">"+
            "<name>Project Manager</name>"+
            "<value>Total</value>"+
          "</column>"+
        "</sub>"+
        "<properties>"+
          "<property name=\"Job Description\">"+
            "<value>Demo Waste Bins</value>"+
          "</property>"+
          "<property name=\"Div\">"+
            "<value>07</value>"+
          "</property>"+
          "<property name=\"Job End Date\">"+
            "<value>Sep 7, 2016</value>"+
          "</property>"+
          "<property name=\"Job Type\">"+
            "<value/>"+
          "</property>"+
          "<property name=\"Job Start Date\">"+
            "<value>Sep 1, 2011</value>"+
          "</property>"+
        "</properties>"+
      "</column>";
		
		Document document = XmlHelper.readXmlDocumentFromString(xmlString);
		Element element = document.getDocumentElement();
		AxisColumn axisColumn = ReportParser.getAxisColumn(element);
		
		assertEquals("GROUPING", axisColumn.getType());
		assertEquals("Job", axisColumn.getName());
		assertEquals("740", axisColumn.getValue());
		assertEquals(2, axisColumn.getSubColumns().size());
		
		List<Property> properties = axisColumn.getProperties();
		
		assertEquals(5, properties.size());
		
		Property property1 = properties.get(0);
		assertEquals("Job Description", property1.getName());
		assertEquals("Demo Waste Bins", property1.getValue());
		
		Property property2 = properties.get(1);
		assertEquals("Div", property2.getName());
		assertEquals("07", property2.getValue());
		
		Property property3 = properties.get(2);
		assertEquals("Job End Date", property3.getName());
		assertEquals("Sep 7, 2016", property3.getValue());
		
		Property property4 = properties.get(3);
		assertEquals("Job Type", property4.getName());
		assertEquals("", property4.getValue());
		
		Property property5 = properties.get(4);
		assertEquals("Job Start Date", property5.getName());
		assertEquals("Sep 1, 2011", property5.getValue());
	}
	
	public void testPreferredJoinEntities() {
		String xmlString = "<preferredJoinEntities>"+
                           "  <entity name=\"iqn.2://indicee.com/10/4/dataModel/entities/customer_id\"/>"+
                           "</preferredJoinEntities>";
		
		Document document = XmlHelper.readXmlDocumentFromString(xmlString);
		Element element = document.getDocumentElement();
		List<String> preferredJoinEntities  = ReportParser.getPreferredJoinEntities(element);
		assertNotNull(preferredJoinEntities);
		assertEquals(1, preferredJoinEntities.size());
		
		assertEquals("iqn.2://indicee.com/10/4/dataModel/entities/customer_id", preferredJoinEntities.get(0));
	}
	
	public void testFilters() {	
		String xml = "<fieldfilters>"+
				"  <fieldfilter entity=\"iqn.2://indicee.com/9/1/dataModel/entities/CommunityName\" field=\"iqn.2://indicee.com/9/1/dataModel/fields/CommunityId\" value=\"&quot;0DBC00000004GeZOAU&quot;\">"+
				"    <attributes>"+
				"      <origin name=\"default\">"+
				"        <attribute name=\"filterDisplayHints\" type=\"stringList\">"+
				"          <item value=\"&quot;0DBC00000004GeZOAU&quot;\"/>"+
				"          <item value=\"SMB Platinum\"/>"+
				"        </attribute>"+
				"        <attribute name=\"userModified\" type=\"boolean\" value=\"true\"/>"+
				"        <attribute name=\"validationStatus\" type=\"string\" value=\"none\"/>"+
				"      </origin>"+
				"    </attributes>"+
				"  </fieldfilter>"+
				"  <relativetimeperiodfilter entity=\"iqn.2://indicee.com/9/1/dataModel/entities/CreatedDateTrim\" field=\"iqn.2://indicee.com/9/1/dataModel/fields/CreatedDateTrim\">"+
				"    <relativetimeperiods>"+
				"      <relativetimeperiod rangeend=\"0\" rangestart=\"0\" timeperiod=\"YearPeriod\"/>"+
				"    </relativetimeperiods>"+
				"    <attributes>"+
				"      <origin name=\"default\">"+
				"        <attribute name=\"userModified\" type=\"boolean\" value=\"true\"/>"+
				"        <attribute name=\"validationStatus\" type=\"string\" value=\"none\"/>"+
				"      </origin>"+
				"    </attributes>"+
				"  </relativetimeperiodfilter>"+
				"</fieldfilters>";
		
		
		Document document = XmlHelper.readXmlDocumentFromString(xml);
		Element element = document.getDocumentElement();
		List<Filter> filters = ReportParser.getFieldFilters(element);
		assertEquals(2, filters.size());
		
		Filter firstFilter = filters.get(0);
		assertTrue(firstFilter.isInclusive());
		
		Filter secondFilter = filters.get(1);
		assertTrue(secondFilter.isInclusive());
	}
}
